import lldb
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


@skipIfNoIntelPT
class TestTraceTimestampCounters(TraceIntelPTTestCaseBase):
    @testSBAPIAndCommands
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testTscPerThread(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartThread(enableTsc=True)

        self.expect("n")
        self.expect(
            "thread trace dump instructions -t -c 1",
            patterns=[r": \[\d+.\d+ ns\] 0x0000000000400511    movl"],
        )

    @testSBAPIAndCommands
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testMultipleTscsPerThread(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartThread(enableTsc=True)

        # After each stop there'll be a new TSC
        self.expect("si")
        self.expect("si")
        self.expect("si")

        # We'll get the most recent instructions, with at least 3 different TSCs
        self.runCmd("thread trace dump instructions -t --raw --forward")
        id_to_timestamp = {}
        for line in self.res.GetOutput().splitlines():
            m = re.search(r"    (.+): \[(.+)\ ns].*", line)
            if m:
                id_to_timestamp[int(m.group(1))] = m.group(2)
        self.assertEqual(len(id_to_timestamp), 3)

        # We check that the values are right when dumping a specific id
        for id, timestamp in id_to_timestamp.items():
            self.expect(
                f"thread trace dump instructions -t --id {id} -c 1",
                substrs=[f"{id}: [{timestamp} ns]"],
            )

    @testSBAPIAndCommands
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testTscPerProcess(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartProcess(enableTsc=True)

        self.expect("n")
        self.expect(
            "thread trace dump instructions -t -c 1",
            patterns=[r": \[\d+.\d+ ns\] 0x0000000000400511    movl"],
        )

        self.expect(
            "thread trace dump instructions -t -c 1 --pretty-json",
            patterns=[r'''"timestamp_ns": "\d+.\d+"'''],
        )

    @testSBAPIAndCommands
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testDumpingAfterTracingWithoutTsc(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartThread(enableTsc=False)

        self.expect("n")
        self.expect(
            "thread trace dump instructions -t -c 1",
            patterns=[r": \[unavailable\] 0x0000000000400511    movl"],
        )

        self.expect(
            "thread trace dump instructions -t -c 1 --json",
            substrs=[""""timestamp_ns":null"""],
        )

    @testSBAPIAndCommands
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testPSBPeriod(self):
        def isPSBSupported():
            caps_file = "/sys/bus/event_source/devices/intel_pt/caps/psb_cyc"
            if not os.path.exists(caps_file):
                return False
            with open(caps_file, "r") as f:
                val = int(f.readline())
                if val != 1:
                    return False
            return True

        def getValidPSBValues():
            values_file = "/sys/bus/event_source/devices/intel_pt/caps/psb_periods"
            values = []
            with open(values_file, "r") as f:
                mask = int(f.readline(), 16)
                for i in range(0, 32):
                    if (1 << i) & mask:
                        values.append(i)
            return values

        if not isPSBSupported():
            self.skipTest("PSB period unsupported")

        valid_psb_values = getValidPSBValues()
        # 0 should always be valid, and it's assumed by lldb-server
        self.assertEqual(valid_psb_values[0], 0)

        self.expect(
            "file " + (os.path.join(self.getSourceDir(), "intelpt-trace", "a.out"))
        )
        self.expect("b main")
        self.expect("r")

        # it's enough to test with two valid values
        for psb_period in (valid_psb_values[0], valid_psb_values[-1]):
            # we first test at thread level
            self.traceStartThread(psbPeriod=psb_period)
            self.traceStopThread()

            # we now test at process level
            self.traceStartProcess(psbPeriod=psb_period)
            self.traceStopProcess()

        # we now test invalid values
        self.traceStartThread(
            psbPeriod=valid_psb_values[-1] + 1,
            error=True,
            substrs=["Invalid psb_period. Valid values are: 0"],
        )

        # TODO: dump the perf_event_attr.config as part of the upcoming "trace dump info"
        # command and check that the psb period is included there.



"""
Test that the save_crashlog command functions
"""


import os
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSaveCrashlog(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_save_crashlog(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.save_crashlog()

    def save_crashlog(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "I was called", self.main_source_file
        )

        self.runCmd("command script import lldb.macosx.crashlog")
        out_file = os.path.join(self.getBuildDir(), "crash.log")
        self.runCmd("save_crashlog '%s'" % (out_file))

        # Make sure we wrote the file:
        self.assertTrue(os.path.exists(out_file), "We wrote our file")

        # Now scan the file to make sure it looks right:
        # First get a few facts we'll use:
        exe_module = target.FindModule(target.GetExecutable())
        uuid_str = exe_module.GetUUIDString()

        # We'll set these to true when we find the elements in the file
        found_call_me = False
        found_main_line = False
        found_thread_header = False
        found_uuid_str = False

        with open(out_file, "r") as f:
            # We want to see a line with
            for line in f:
                if "Thread 0:" in line:
                    found_thread_header = True
                if "call_me" in line and "main.c:" in line:
                    found_call_me = True
                if "main" in line and "main.c:" in line:
                    found_main_line = True
                if uuid_str in line and "a.out" in line:
                    found_uuid_str = True

        self.assertTrue(found_thread_header, "Found thread header")
        self.assertTrue(found_call_me, "Found call_me line in stack")
        self.assertTrue(found_uuid_str, "Found main binary UUID")
        self.assertTrue(found_main_line, "Found main line in call stack")



"""
Test python scripted process in lldb
"""

import os, json, tempfile

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test import lldbtest


class StackCoreScriptedProcesTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def create_stack_skinny_corefile(self, file):
        self.build()
        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("baz.cpp")
        )
        self.assertTrue(process.IsValid(), "Process is invalid.")
        # FIXME: Use SBAPI to save the process corefile.
        self.runCmd("process save-core -s stack  " + file)
        self.assertTrue(os.path.exists(file), "No stack-only corefile found.")
        self.assertTrue(self.dbg.DeleteTarget(target), "Couldn't delete target")

    def get_module_with_name(self, target, name):
        for module in target.modules:
            if name in module.GetFileSpec().GetFilename():
                return module
        return None

    @skipUnlessDarwin
    @skipIfRemote
    @skipIfAsan  # On ASAN builds, this test times-out (rdar://98678134)
    @skipIfDarwin
    def test_launch_scripted_process_stack_frames(self):
        """Test that we can launch an lldb scripted process from the command
        line, check its process ID and read string from memory."""
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        main_module = self.get_module_with_name(target, "a.out")
        self.assertTrue(main_module, "Invalid main module.")
        error = target.SetModuleLoadAddress(main_module, 0)
        self.assertSuccess(error, "Reloading main module at offset 0 failed.")

        scripted_dylib = self.get_module_with_name(target, "libbaz.dylib")
        self.assertTrue(scripted_dylib, "Dynamic library libbaz.dylib not found.")
        self.assertEqual(
            scripted_dylib.GetObjectFileHeaderAddress().GetLoadAddress(target),
            0xFFFFFFFFFFFFFFFF,
        )

        os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"] = "1"

        def cleanup():
            del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]

        self.addTearDownHook(cleanup)

        scripted_process_example_relpath = "stack_core_scripted_process.py"
        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), scripted_process_example_relpath)
        )

        corefile_process = None
        with tempfile.NamedTemporaryFile() as file:
            self.create_stack_skinny_corefile(file.name)
            corefile_target = self.dbg.CreateTarget(None)
            corefile_process = corefile_target.LoadCore(
                self.getBuildArtifact(file.name)
            )
        self.assertTrue(corefile_process, PROCESS_IS_VALID)

        # Create a random lib which does not exist in the corefile.
        random_dylib = self.get_module_with_name(corefile_target, "random.dylib")
        self.assertFalse(
            random_dylib, "Dynamic library random.dylib should not be found."
        )

        structured_data = lldb.SBStructuredData()
        structured_data.SetFromJSON(
            json.dumps(
                {
                    "backing_target_idx": self.dbg.GetIndexOfTarget(
                        corefile_process.GetTarget()
                    ),
                    "custom_modules": [
                        {
                            "path": self.getBuildArtifact("libbaz.dylib"),
                        },
                        {
                            "path": "/random/path/random.dylib",
                            "load_addr": 12345678,
                        },
                    ],
                }
            )
        )
        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName(
            "stack_core_scripted_process.StackCoreScriptedProcess"
        )
        launch_info.SetScriptedProcessDictionary(structured_data)

        error = lldb.SBError()
        process = target.Launch(launch_info, error)
        self.assertSuccess(error)
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 42)

        self.assertEqual(process.GetNumThreads(), 2)
        thread = process.GetSelectedThread()
        self.assertTrue(thread, "Invalid thread.")
        self.assertEqual(thread.GetName(), "StackCoreScriptedThread.thread-1")

        self.assertTrue(target.triple, "Invalid target triple")
        arch = target.triple.split("-")[0]
        supported_arch = ["x86_64", "arm64", "arm64e"]
        self.assertIn(arch, supported_arch)
        # When creating a corefile of a arm process, lldb saves the exception
        # that triggers the breakpoint in the LC_NOTES of the corefile, so they
        # can be reloaded with the corefile on the next debug session.
        if arch in "arm64e":
            self.assertTrue(thread.GetStopReason(), lldb.eStopReasonException)
        # However, it's architecture specific, and corefiles made from intel
        # process don't save any metadata to retrieve to stop reason.
        # To mitigate this, the StackCoreScriptedProcess will report a
        # eStopReasonSignal with a SIGTRAP, mimicking what debugserver does.
        else:
            self.assertTrue(thread.GetStopReason(), lldb.eStopReasonSignal)

        self.assertEqual(thread.GetNumFrames(), 5)
        frame = thread.GetSelectedFrame()
        self.assertTrue(frame, "Invalid frame.")
        func = frame.GetFunction()
        self.assertTrue(func, "Invalid function.")

        self.assertIn("baz", frame.GetFunctionName())
        self.assertGreater(frame.vars.GetSize(), 0)
        self.assertEqual(int(frame.vars.GetFirstValueByName("k").GetValue()), 42)
        self.assertEqual(
            int(frame.vars.GetFirstValueByName("j").Dereference().GetValue()), 42 * 42
        )

        corefile_dylib = self.get_module_with_name(corefile_target, "libbaz.dylib")
        self.assertTrue(corefile_dylib, "Dynamic library libbaz.dylib not found.")
        scripted_dylib = self.get_module_with_name(target, "libbaz.dylib")
        self.assertTrue(scripted_dylib, "Dynamic library libbaz.dylib not found.")
        self.assertEqual(
            scripted_dylib.GetObjectFileHeaderAddress().GetLoadAddress(target),
            corefile_dylib.GetObjectFileHeaderAddress().GetLoadAddress(target),
        )



"""Test evaluating expressions which ref. index variable 'i' which just goes
from out of scope to in scope when stopped at the breakpoint."""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class NonOverlappingIndexVariableCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.cpp"
        self.line_to_break = line_number(self.source, "// Set breakpoint here.")

    # rdar://problem/9890530
    def test_eval_index_variable(self):
        """Test expressions of variable 'i' which appears in two for loops."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            self.source,
            self.line_to_break,
            num_expected_locations=1,
            loc_exact=True,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.runCmd("frame variable i")
        self.runCmd("expr i")
        self.runCmd("expr ptr[0]->point.x")
        self.runCmd("expr ptr[0]->point.y")
        self.runCmd("expr ptr[i]->point.x")
        self.runCmd("expr ptr[i]->point.y")



import lldb


class Resolver:
    got_files = 0
    func_list = []

    def __init__(self, bkpt, extra_args, dict):
        self.bkpt = bkpt
        self.extra_args = extra_args

        Resolver.func_list = []
        Resolver.got_files = 0

    def __callback__(self, sym_ctx):
        print("Resolver callback called")
        sym_name = "not_a_real_function_name"
        sym_item = self.extra_args.GetValueForKey("symbol")
        if sym_item.IsValid():
            sym_name = sym_item.GetStringValue(1000)
        else:
            print("Didn't have a value for key 'symbol'")

        if sym_ctx.compile_unit.IsValid():
            Resolver.got_files = 1
        else:
            Resolver.got_files = 2

        if sym_ctx.function.IsValid():
            Resolver.got_files = 3
            func_name = sym_ctx.function.GetName()
            Resolver.func_list.append(func_name)
            if sym_name == func_name:
                self.bkpt.AddLocation(sym_ctx.function.GetStartAddress())
            return

        if sym_ctx.module.IsValid():
            print(f"Looking for {sym_name}")
            sym = sym_ctx.module.FindSymbol(sym_name, lldb.eSymbolTypeCode)
            if sym.IsValid():
                print(f"Adding location at {sym.GetStartAddress()} to {self.bkpt}")
                self.bkpt.AddLocation(sym.GetStartAddress())
                print(f"After addition: {self.bkpt}")
            else:
                print("Didn't find it, however...")

        print(f"GotFiles: {Resolver.got_files}")
        for func in Resolver.func_list:
            print(f"Function: func")

    def get_short_help(self):
        return "I am a python breakpoint resolver"

class ResolverModuleDepth(Resolver):
    def __get_depth__(self):
        return lldb.eSearchDepthModule

    def __callback__(self, sym_ctx):
        print(f"About to call the Resolver callback for {self.bkpt}")
        Resolver.__callback__(self, sym_ctx)
        print("Called the callback for ResolverModuleDepth")


class ResolverCUDepth(Resolver):
    def __get_depth__(self):
        return lldb.eSearchDepthCompUnit

    def __callback__(self, sym_ctx):
        Resolver.__callback__(self, sym_ctx)


class ResolverFuncDepth(Resolver):
    def __get_depth__(self):
        return lldb.eSearchDepthFunction

    def __callback__(self, sym_ctx):
        Resolver.__callback__(self, sym_ctx)


class ResolverBadDepth(Resolver):
    def __get_depth__(self):
        return lldb.kLastSearchDepthKind + 1

    def __callback__(self, sym_ctx):
        Resolver.__callback__(self, sym_ctx)



"""
Test jumping to different places.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ThreadJumpTestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.build()

    def test(self):
        """Test thread jump handling."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Find the line numbers for our breakpoints.
        self.mark1 = line_number("main.cpp", "// 1st marker")
        self.mark2 = line_number("main.cpp", "// 2nd marker")
        self.mark3 = line_number("main.cpp", "// 3rd marker")
        self.mark4 = line_number("main.cpp", "// 4th marker")
        self.mark5 = line_number("other.cpp", "// other marker")

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.mark3, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint 1.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT + " 1",
            substrs=[
                "stopped",
                "main.cpp:{}".format(self.mark3),
                "stop reason = breakpoint 1",
            ],
        )

        # Try the int path, force it to return 'a'
        self.do_min_test(self.mark3, self.mark1, "i", "4")
        # Try the int path, force it to return 'b'
        self.do_min_test(self.mark3, self.mark2, "i", "5")
        # Try the double path, force it to return 'a'
        self.do_min_test(self.mark4, self.mark1, "j", "7")
        # Expected to fail on powerpc64le architecture
        if not self.isPPC64le():
            # Try the double path, force it to return 'b'
            self.do_min_test(self.mark4, self.mark2, "j", "8")

        # Try jumping to another function in a different file.
        self.runCmd("thread jump --file other.cpp --line %i --force" % self.mark5)
        self.expect("process status", substrs=["at other.cpp:%i" % self.mark5])

        # Try jumping to another function (without forcing)
        self.expect(
            "j main.cpp:%i" % self.mark1,
            COMMAND_FAILED_AS_EXPECTED,
            error=True,
            substrs=["error"],
        )

    @expectedFailureAll(compiler="clang", compiler_version=["<", "17.0"])
    def test_jump_offset(self):
        """Test Thread Jump by negative or positive offset"""
        exe = self.getBuildArtifact("a.out")
        file_name = "main.cpp"
        self.runCmd(f"target create {exe}", CURRENT_EXECUTABLE_SET)

        pos_jump = line_number(file_name, "// jump_offset 1")
        neg_jump = line_number(file_name, "// jump_offset 2")
        pos_breakpoint = line_number(file_name, "// breakpoint 1")
        neg_breakpoint = line_number(file_name, "// breakpoint 2")
        pos_jump_offset = pos_jump - pos_breakpoint
        neg_jump_offset = neg_jump - neg_breakpoint

        var_1, var_1_value = ("var_1", "10")
        var_2, var_2_value = ("var_2", "40")
        var_3, var_3_value = ("var_3", "10")

        # create pos_breakpoint and neg_breakpoint
        lldbutil.run_break_set_by_file_and_line(
            self, file_name, pos_breakpoint, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, file_name, neg_breakpoint, num_expected_locations=1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # test positive jump
        # The stop reason of the thread should be breakpoint 1.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT + " 1",
            substrs=[
                "stopped",
                f"{file_name}:{pos_breakpoint}",
                "stop reason = breakpoint 1",
            ],
        )

        self.runCmd(f"thread jump --by +{pos_jump_offset}")
        self.expect("process status", substrs=[f"at {file_name}:{pos_jump}"])
        self.expect(f"print {var_1}", substrs=[var_1_value])

        self.runCmd("thread step-over")
        self.expect(f"print {var_2}", substrs=[var_2_value])

        self.runCmd("continue")

        # test negative jump
        # The stop reason of the thread should be breakpoint 1.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT + " 2",
            substrs=[
                "stopped",
                f"{file_name}:{neg_breakpoint}",
                "stop reason = breakpoint 2",
            ],
        )

        self.runCmd(f"thread jump --by {neg_jump_offset}")
        self.expect("process status", substrs=[f"at {file_name}:{neg_jump}"])
        self.runCmd("thread step-over")
        self.expect(f"print {var_3}", substrs=[var_3_value])

    def do_min_test(self, start, jump, var, value):
        # jump to the start marker
        self.runCmd("j %i" % start)
        self.runCmd("thread step-in")  # step into the min fn
        # jump to the branch we're interested in
        self.runCmd("j %i" % jump)
        self.runCmd("thread step-out")  # return out
        self.runCmd("thread step-over")  # assign to the global
        self.expect("expr %s" % var, substrs=[value])  # check it



# Summaries for common ObjC types that require Python scripting
# to be generated fit into this file


def BOOL_SummaryProvider(valobj, dict):
    if not (valobj.IsValid()):
        return "<invalid>"
    if valobj.GetValueAsUnsigned() == 0:
        return "NO"
    else:
        return "YES"


def BOOLRef_SummaryProvider(valobj, dict):
    return BOOL_SummaryProvider(valobj.GetChildAtIndex(0), dict)


def BOOLPtr_SummaryProvider(valobj, dict):
    return BOOL_SummaryProvider(valobj.Dereference(), dict)



"""Test that the expedited thread pc values are not re-fetched by lldb."""

import subprocess
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

file_index = 0


class TestExpeditedThreadPCs(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_expedited_thread_pcs(self):
        TestBase.setUp(self)

        global file_index
        ++file_index
        logfile = os.path.join(
            self.getBuildDir(),
            "packet-log-" + self.getArchitecture() + "-" + str(file_index) + ".txt",
        )
        self.runCmd("log enable -f %s gdb-remote packets" % (logfile))

        def cleanup():
            self.runCmd("log disable gdb-remote packets")
            if os.path.exists(logfile):
                os.unlink(logfile)

        self.addTearDownHook(cleanup)

        self.source = "main.cpp"
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source, False)
        )

        # verify that libfoo.dylib hasn't loaded yet
        for m in target.modules:
            self.assertNotEqual(m.GetFileSpec().GetFilename(), "libfoo.dylib")

        thread.StepInto()
        thread.StepInto()

        thread.StepInto()
        thread.StepInto()
        thread.StepInto()

        # verify that libfoo.dylib has loaded
        for m in target.modules:
            if m.GetFileSpec().GetFilename() == "libfoo.dylib":
                found_libfoo = True
        self.assertTrue(found_libfoo)

        thread.StepInto()
        thread.StepInto()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()
        thread.StepOver()

        process.Kill()

        # Confirm that we never fetched the pc for any threads during
        # this debug session.
        if os.path.exists(logfile):
            f = open(logfile)
            lines = f.readlines()
            num_errors = 0
            for line in lines:
                arch = self.getArchitecture()
                if arch == "arm64" or arch == "arm64_32":
                    #   <reg name="pc" regnum="32" offset="256" bitsize="64" group="general" group_id="1" ehframe_regnum="32" dwarf_regnum="32" generic="pc"/>
                    # A fetch of $pc on arm64 looks like
                    #  <  22> send packet: $p20;thread:91698e;#70
                    self.assertNotIn("$p20;thread", line)
                else:
                    #   <reg name="rip" regnum="16" offset="128" bitsize="64" group="general" altname="pc" group_id="1" ehframe_regnum="16" dwarf_regnum="16" generic="pc"/>
                    # A fetch of $pc on x86_64 looks like
                    #  <  22> send packet: $p10;thread:91889c;#6f
                    self.assertNotIn("$p10;thread", line)

            f.close()



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg

from mlir.dialects.linalg.opdsl.lang import *

T1 = TV.T1
T2 = TV.T2


@linalg_structured_op
def fill_poly(value=ScalarDef(T1), O=TensorDef(U, output=True)):
    O[None] = TypeFn.cast_signed(U, value)


@linalg_structured_op
def fill_rank_zero_poly(I=TensorDef(T1), O=TensorDef(U, output=True)):
    O[None] = TypeFn.cast_signed(U, I[None])


with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):

        # Fill indexing maps.
        # CHECK-DAG: #[[$MAP0:.+]] = affine_map<() -> ()>
        # CHECK-DAG: #[[$MAP1:.+]] = affine_map<(d0, d1) -> ()>
        # CHECK-DAG: #[[$MAP2:.+]] = affine_map<(d0, d1) -> (d0, d1)>
        # CHECK-DAG: #[[$MAP3:.+]] = affine_map<(d0, d1, d2) -> ()>
        # CHECK-DAG: #[[$MAP4:.+]] = affine_map<(d0, d1, d2) -> (d0, d1, d2)>

        # CHECK-LABEL: @test_fill_0d
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$MAP0]], #[[$MAP0]]
        # CHECK-SAME: iterator_types = []
        @func.FuncOp.from_py_func(f32, RankedTensorType.get([], f32))
        def test_fill_0d(value, init_result):
            return fill_poly(value, outs=[init_result])

        # CHECK-LABEL: @test_fill_2d
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$MAP1]], #[[$MAP2]]]
        # CHECK-SAME: iterator_types = ["parallel", "parallel"]
        @func.FuncOp.from_py_func(f32, RankedTensorType.get([4, 16], f32))
        def test_fill_2d(value, init_result):
            return fill_poly(value, outs=[init_result])

        # CHECK-LABEL: @test_fill_rank_zero_3d
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$MAP3]], #[[$MAP4]]]
        # CHECK-SAME: iterator_types = ["parallel", "parallel", "parallel"]
        @func.FuncOp.from_py_func(
            RankedTensorType.get([], f32), RankedTensorType.get([4, 8, 16], f32)
        )
        def test_fill_rank_zero_3d(input, init_result):
            return fill_rank_zero_poly(input, outs=[init_result])


print(module)



"""Test stepping over and into inlined functions."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestInlineStepping(TestBase):
    @add_test_categories(["pyapi"])
    @skipIf(oslist=["windows"], archs=["aarch64"])  # Flaky on buildbot
    @expectedFailureAll(
        compiler="icc",
        bugnumber="# Not really a bug.  ICC combines two inlined functions.",
    )
    def test_with_python_api(self):
        """Test stepping over and into inlined functions."""
        self.build()
        self.inline_stepping()

    @add_test_categories(["pyapi"])
    @skipIf(oslist=["windows"], archs=["aarch64"])  # Flaky on buildbot
    def test_step_over_with_python_api(self):
        """Test stepping over and into inlined functions."""
        self.build()
        self.inline_stepping_step_over()

    @add_test_categories(["pyapi"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr32343")
    def test_step_in_template_with_python_api(self):
        """Test stepping in to templated functions."""
        self.build()
        self.step_in_template()

    @add_test_categories(["pyapi"])
    def test_virtual_inline_stepping(self):
        """Test stepping through a virtual inlined call stack"""
        self.build()
        self.virtual_inline_stepping()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "calling.cpp"
        self.source_lines = {}
        functions = [
            "caller_ref_1",
            "caller_ref_2",
            "inline_ref_1",
            "inline_ref_2",
            "called_by_inline_ref",
            "caller_trivial_1",
            "caller_trivial_2",
            "inline_trivial_1",
            "inline_trivial_2",
            "called_by_inline_trivial",
        ]
        for name in functions:
            self.source_lines[name] = line_number(
                self.main_source, "// In " + name + "."
            )
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_step(self, step_type, destination_line_entry, test_stack_depth):
        expected_stack_depth = self.thread.GetNumFrames()
        if step_type == "into":
            expected_stack_depth += 1
            self.thread.StepInto()
        elif step_type == "out":
            expected_stack_depth -= 1
            self.thread.StepOut()
        elif step_type == "over":
            self.thread.StepOver()
        else:
            self.fail("Unrecognized step type: " + step_type)

        threads = lldbutil.get_stopped_threads(
            self.process, lldb.eStopReasonPlanComplete
        )
        if len(threads) != 1:
            destination_description = lldb.SBStream()
            destination_line_entry.GetDescription(destination_description)
            self.fail(
                "Failed to stop due to step "
                + step_type
                + " operation stepping to: "
                + destination_description.GetData()
            )

        self.thread = threads[0]

        stop_line_entry = self.thread.GetFrameAtIndex(0).GetLineEntry()
        self.assertTrue(stop_line_entry.IsValid(), "Stop line entry was not valid.")

        # Don't use the line entry equal operator because we don't care about
        # the column number.
        stop_at_right_place = (
            stop_line_entry.GetFileSpec() == destination_line_entry.GetFileSpec()
            and stop_line_entry.GetLine() == destination_line_entry.GetLine()
        )
        if not stop_at_right_place:
            destination_description = lldb.SBStream()
            destination_line_entry.GetDescription(destination_description)

            actual_description = lldb.SBStream()
            stop_line_entry.GetDescription(actual_description)

            self.fail(
                "Step "
                + step_type
                + " stopped at wrong place: expected: "
                + destination_description.GetData()
                + " got: "
                + actual_description.GetData()
                + "."
            )

        real_stack_depth = self.thread.GetNumFrames()

        if test_stack_depth and real_stack_depth != expected_stack_depth:
            destination_description = lldb.SBStream()
            destination_line_entry.GetDescription(destination_description)
            self.fail(
                "Step %s to %s got wrong number of frames, should be: %d was: %d."
                % (
                    step_type,
                    destination_description.GetData(),
                    expected_stack_depth,
                    real_stack_depth,
                )
            )

    def run_step_sequence(self, step_sequence):
        """This function takes a list of duples instructing how to run the program.  The first element in each duple is
        a source pattern for the target location, and the second is the operation that will take you from the current
        source location to the target location.  It will then run all the steps in the sequence.
        It will check that you arrived at the expected source location at each step, and that the stack depth changed
        correctly for the operation in the sequence."""

        target_line_entry = lldb.SBLineEntry()
        target_line_entry.SetFileSpec(self.main_source_spec)

        test_stack_depth = True
        # Work around for <rdar://problem/16363195>, the darwin unwinder seems flakey about whether it duplicates the first frame
        # or not, which makes counting stack depth unreliable.
        if self.platformIsDarwin():
            test_stack_depth = False

        for step_pattern in step_sequence:
            step_stop_line = line_number(self.main_source, step_pattern[0])
            target_line_entry.SetLine(step_stop_line)
            self.do_step(step_pattern[1], target_line_entry, test_stack_depth)

    def inline_stepping(self):
        """Use Python APIs to test stepping over and hitting breakpoints."""
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        break_1_in_main = target.BreakpointCreateBySourceRegex(
            "// Stop here and step over to set up stepping over.", self.main_source_spec
        )
        self.assertTrue(break_1_in_main, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        self.process = target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )

        self.assertTrue(self.process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, break_1_in_main
        )

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        self.thread = threads[0]

        # Step over the inline_value = 0 line to get us to inline_trivial_1 called from main.  Doing it this way works
        # around a bug in lldb where the breakpoint on the containing line of an inlined function with no return value
        # gets set past the insertion line in the function.
        # Then test stepping over a simple inlined function.  Note, to test all the parts of the inlined stepping
        # the calls inline_stepping_1 and inline_stepping_2 should line up at the same address, that way we will test
        # the "virtual" stepping.
        # FIXME: Put in a check to see if that is true and warn if it is not.

        step_sequence = [
            ["// At inline_trivial_1 called from main.", "over"],
            ["// At first call of caller_trivial_1 in main.", "over"],
        ]
        self.run_step_sequence(step_sequence)

        # Now step from caller_ref_1 all the way into called_by_inline_trivial

        step_sequence = [
            ["// In caller_trivial_1.", "into"],
            ["// In caller_trivial_2.", "into"],
            ["// In inline_trivial_1.", "into"],
            ["// In inline_trivial_2.", "into"],
            ["// At caller_by_inline_trivial in inline_trivial_2.", "over"],
            ["// In called_by_inline_trivial.", "into"],
        ]
        self.run_step_sequence(step_sequence)

        # Now run to the inline_trivial_1 just before the immediate step into
        # inline_trivial_2:

        break_2_in_main = target.BreakpointCreateBySourceRegex(
            "// At second call of caller_trivial_1 in main.", self.main_source_spec
        )
        self.assertTrue(break_2_in_main, VALID_BREAKPOINT)

        threads = lldbutil.continue_to_breakpoint(self.process, break_2_in_main)
        self.assertEqual(
            len(threads),
            1,
            "Successfully ran to call site of second caller_trivial_1 call.",
        )
        self.thread = threads[0]

        step_sequence = [
            ["// In caller_trivial_1.", "into"],
            ["// In caller_trivial_2.", "into"],
            ["// In inline_trivial_1.", "into"],
        ]
        self.run_step_sequence(step_sequence)

        # Then call some trivial function, and make sure we end up back where
        # we were in the inlined call stack:

        frame = self.thread.GetFrameAtIndex(0)
        before_line_entry = frame.GetLineEntry()
        value = frame.EvaluateExpression("function_to_call()")
        after_line_entry = frame.GetLineEntry()

        self.assertEqual(
            before_line_entry.GetLine(),
            after_line_entry.GetLine(),
            "Line entry before and after function calls are the same.",
        )

        # Now make sure stepping OVER in the middle of the stack works, and
        # then check finish from the inlined frame:

        step_sequence = [
            ["// At increment in inline_trivial_1.", "over"],
            ["// At increment in caller_trivial_2.", "out"],
        ]
        self.run_step_sequence(step_sequence)

        # Now run to the place in main just before the first call to
        # caller_ref_1:

        break_3_in_main = target.BreakpointCreateBySourceRegex(
            "// At first call of caller_ref_1 in main.", self.main_source_spec
        )
        self.assertTrue(break_3_in_main, VALID_BREAKPOINT)

        threads = lldbutil.continue_to_breakpoint(self.process, break_3_in_main)
        self.assertEqual(
            len(threads), 1, "Successfully ran to call site of first caller_ref_1 call."
        )
        self.thread = threads[0]

        step_sequence = [
            ["// In caller_ref_1.", "into"],
            ["// In caller_ref_2.", "into"],
            ["// In inline_ref_1.", "into"],
            ["// In inline_ref_2.", "into"],
            ["// In called_by_inline_ref.", "into"],
            ["// In inline_ref_2.", "out"],
            ["// In inline_ref_1.", "out"],
            ["// At increment in inline_ref_1.", "over"],
            ["// In caller_ref_2.", "out"],
            ["// At increment in caller_ref_2.", "over"],
        ]
        self.run_step_sequence(step_sequence)

    def inline_stepping_step_over(self):
        """Use Python APIs to test stepping over and hitting breakpoints."""
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        break_1_in_main = target.BreakpointCreateBySourceRegex(
            "// At second call of caller_ref_1 in main.", self.main_source_spec
        )
        self.assertGreater(break_1_in_main.GetNumLocations(), 0, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        self.process = target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )

        self.assertTrue(self.process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, break_1_in_main
        )

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        self.thread = threads[0]

        step_sequence = [
            ["// In caller_ref_1.", "into"],
            ["// In caller_ref_2.", "into"],
            ["// At increment in caller_ref_2.", "over"],
        ]
        self.run_step_sequence(step_sequence)

        # Now make sure that next to a virtual inlined call stack
        # gets the call stack depth correct.
        break_2_in_main = target.BreakpointCreateBySourceRegex(
            "// Call max_value specialized", self.main_source_spec
        )
        self.assertGreater(break_2_in_main.GetNumLocations(), 0, VALID_BREAKPOINT)
        threads = lldbutil.continue_to_breakpoint(self.process, break_2_in_main)
        self.assertEqual(len(threads), 1, "Hit our second breakpoint")
        self.assertEqual(threads[0].id, self.thread.id, "Stopped at right thread")
        self.thread.StepOver()
        frame_0 = self.thread.frames[0]
        line_entry = frame_0.line_entry
        self.assertEqual(
            line_entry.file.basename, self.main_source_spec.basename, "File matches"
        )
        target_line = line_number("calling.cpp", "// At caller_trivial_inline_1")
        self.assertEqual(line_entry.line, target_line, "Lines match as well.")

    def step_in_template(self):
        """Use Python APIs to test stepping in to templated functions."""
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        break_1_in_main = target.BreakpointCreateBySourceRegex(
            "// Call max_value template", self.main_source_spec
        )
        self.assertTrue(break_1_in_main, VALID_BREAKPOINT)

        break_2_in_main = target.BreakpointCreateBySourceRegex(
            "// Call max_value specialized", self.main_source_spec
        )
        self.assertTrue(break_2_in_main, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        self.process = target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(self.process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, break_1_in_main
        )

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        self.thread = threads[0]

        step_sequence = [["// In max_value template", "into"]]
        self.run_step_sequence(step_sequence)

        threads = lldbutil.continue_to_breakpoint(self.process, break_2_in_main)
        self.assertEqual(
            len(threads),
            1,
            "Successfully ran to call site of second caller_trivial_1 call.",
        )
        self.thread = threads[0]

        step_sequence = [["// In max_value specialized", "into"]]
        self.run_step_sequence(step_sequence)

    def run_to_call_site_and_step(
        self, source_regex, func_name, start_pos, one_more_step_loc=None
    ):
        main_spec = lldb.SBFileSpec("calling.cpp")
        # Set the breakpoint by file and line, not sourced regex because
        # we want to make sure we can set breakpoints on call sites:
        call_site_line_num = line_number(self.main_source, source_regex)
        target, process, thread, bkpt = lldbutil.run_to_line_breakpoint(
            self, main_spec, call_site_line_num
        )

        # Make sure that the location is at the call site (run_to_line_breakpoint already asserted
        # that there's one location.):
        bkpt_loc = bkpt.location[0]
        strm = lldb.SBStream()
        result = bkpt_loc.GetDescription(strm, lldb.eDescriptionLevelFull)

        self.assertTrue(result, "Got a location description")
        desc = strm.GetData()
        self.assertIn(f"calling.cpp:{call_site_line_num}", desc, "Right line listed")
        # We don't get the function name right yet - so we omit it in printing.
        # Turn on this test when that is working.
        # self.assertIn(func_name, desc, "Right function listed")

        pc = thread.frame[0].pc
        for i in range(start_pos, 3):
            thread.StepInto()
            frame_0 = thread.frame[0]

            trivial_line_num = line_number(
                self.main_source, f"In caller_trivial_inline_{i}."
            )
            self.assertEqual(
                frame_0.line_entry.line,
                trivial_line_num,
                f"Stepped into the caller_trivial_inline_{i}",
            )
            if pc != frame_0.pc:
                # If we get here, we stepped to the expected line number, but
                # the compiler on this system has decided to insert an instruction
                # between the call site of an inlined function with no arguments,
                # returning void, and its immediate call to another void inlined function
                # with no arguments.  We aren't going to be testing virtual inline
                # stepping for this function...
                break

        if one_more_step_loc:
            thread.StepInto()
            frame_0 = thread.frame[0]
            self.assertEqual(
                frame_0.line_entry.line,
                line_number(self.main_source, one_more_step_loc),
                "Was able to step one more time",
            )
        process.Kill()
        target.Clear()

    def virtual_inline_stepping(self):
        """Use the Python API's to step through a virtual inlined stack"""
        self.run_to_call_site_and_step("At caller_trivial_inline_1", "main", 1)
        self.run_to_call_site_and_step(
            "In caller_trivial_inline_1", "caller_trivial_inline_1", 2
        )
        self.run_to_call_site_and_step(
            "In caller_trivial_inline_2", "caller_trivial_inline_2", 3
        )
        self.run_to_call_site_and_step(
            "In caller_trivial_inline_3",
            "caller_trivial_inline_3",
            4,
            "After caller_trivial_inline_3",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class OdrHandlingWithDylibTestCase(TestBase):
    @skipIf(
        bugnumber="https://github.com/llvm/llvm-project/issues/50375, rdar://135551810"
    )
    def test(self):
        """
        Tests that the expression evaluator is able to deal with types
        whose definitions conflict across multiple LLDB modules (in this
        case the definition for 'class Service' in the main executable
        has an additional field compared to the definition found in the
        dylib). This causes the ASTImporter to detect a name conflict
        while importing 'Service'. With LLDB's liberal ODRHandlingType
        the ASTImporter happily creates a conflicting AST node for
        'Service' in the scratch ASTContext, leading to a crash down
        the line.
        """
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "plugin_entry", lldb.SBFileSpec("plugin.cpp")
        )

        self.expect_expr("*gProxyThis")



"""
Test some SBModule and SBSection APIs.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbutil import symbol_type_to_str


class ModuleAndSectionAPIsTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def test_module_and_section(self):
        """Test module and section APIs."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.assertGreater(target.GetNumModules(), 0)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print("Number of modules for the target: %d" % target.GetNumModules())
        for module in target.module_iter():
            print(module)

        # Get the executable module at index 0.
        exe_module = target.GetModuleAtIndex(0)

        print("Exe module: %s" % str(exe_module))
        print("Number of sections: %d" % exe_module.GetNumSections())
        print("Number of symbols: %d" % len(exe_module))
        INDENT = " " * 4
        INDENT2 = INDENT * 2
        for sec in exe_module.section_iter():
            print(sec)
            print(INDENT + "Number of subsections: %d" % sec.GetNumSubSections())
            if sec.GetNumSubSections() == 0:
                for sym in exe_module.symbol_in_section_iter(sec):
                    print(INDENT + str(sym))
                    print(
                        INDENT + "symbol type: %s" % symbol_type_to_str(sym.GetType())
                    )
            else:
                for subsec in sec:
                    print(INDENT + str(subsec))
                    # Now print the symbols belonging to the subsection....
                    for sym in exe_module.symbol_in_section_iter(subsec):
                        print(INDENT2 + str(sym))
                        print(
                            INDENT2
                            + "symbol type: %s" % symbol_type_to_str(sym.GetType())
                        )

    def test_module_and_section_boundary_condition(self):
        """Test module and section APIs by passing None when it expects a Python string."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.assertGreater(target.GetNumModules(), 0)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print("Number of modules for the target: %d" % target.GetNumModules())
        for module in target.module_iter():
            print(module)

        # Get the executable module at index 0.
        exe_module = target.GetModuleAtIndex(0)

        print("Exe module: %s" % str(exe_module))
        print("Number of sections: %d" % exe_module.GetNumSections())

        # Boundary condition testings.  Should not crash lldb!
        exe_module.FindFirstType(None)
        exe_module.FindTypes(None)
        exe_module.FindGlobalVariables(target, None, 1)
        exe_module.FindFunctions(None, 0)
        exe_module.FindSection(None)

        # Get the section at index 1.
        if exe_module.GetNumSections() > 1:
            sec1 = exe_module.GetSectionAtIndex(1)
            print(sec1)
        else:
            sec1 = None

        if sec1:
            sec1.FindSubSection(None)

    def test_module_compile_unit_iter(self):
        """Test module's compile unit iterator APIs."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.assertGreater(target.GetNumModules(), 0)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print("Number of modules for the target: %d" % target.GetNumModules())
        for module in target.module_iter():
            print(module)

        # Get the executable module at index 0.
        exe_module = target.GetModuleAtIndex(0)

        print("Exe module: %s" % str(exe_module))
        print("Number of compile units: %d" % exe_module.GetNumCompileUnits())
        INDENT = " " * 4
        INDENT2 = INDENT * 2
        for cu in exe_module.compile_unit_iter():
            print(cu)

    def test_find_compile_units(self):
        """Exercise SBModule.FindCompileUnits() API."""
        d = {"EXE": "b.out"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        self.find_compile_units(self.getBuildArtifact("b.out"))

    def find_compile_units(self, exe):
        """Exercise SBModule.FindCompileUnits() API."""
        source_name_list = ["main.cpp", "b.cpp", "c.cpp"]

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        num_modules = target.GetNumModules()
        for i in range(num_modules):
            module = target.GetModuleAtIndex(i)
            for source_name in source_name_list:
                list = module.FindCompileUnits(lldb.SBFileSpec(source_name, False))
                for sc in list:
                    self.assertEqual(
                        sc.GetCompileUnit().GetFileSpec().GetFilename(), source_name
                    )



"""
Test "memory tag read" and "memory tag write" commands
on AArch64 Linux with MTE.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxMTEMemoryTagAccessTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setup_mte_test(self):
        if not self.isAArch64MTE():
            self.skipTest("Target must support MTE.")

        # Required to check that commands remove non-address bits
        # other than the memory tags.
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Breakpoint here"),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_tag_read(self):
        self.setup_mte_test()

        # Argument validation
        self.expect(
            "memory tag read",
            substrs=[
                "error: wrong number of arguments; expected at least <address-expression>, "
                "at most <address-expression> <end-address-expression>"
            ],
            error=True,
        )
        self.expect(
            "memory tag read mte_buf buf+16 32",
            substrs=[
                "error: wrong number of arguments; expected at least <address-expression>, "
                "at most <address-expression> <end-address-expression>"
            ],
            error=True,
        )
        self.expect(
            "memory tag read not_a_symbol",
            substrs=[
                'error: Invalid address expression, address expression "not_a_symbol" '
                "evaluation failed"
            ],
            error=True,
        )
        self.expect(
            "memory tag read mte_buf not_a_symbol",
            substrs=[
                'error: Invalid end address expression, address expression "not_a_symbol" '
                "evaluation failed"
            ],
            error=True,
        )
        # Inverted range
        self.expect(
            "memory tag read mte_buf mte_buf-16",
            patterns=[
                r"error: End address \(0x[A-Fa-f0-9]+\) must be "
                r"greater than the start address \(0x[A-Fa-f0-9]+\)"
            ],
            error=True,
        )
        # Range of length 0
        self.expect(
            "memory tag read mte_buf mte_buf",
            patterns=[
                r"error: End address \(0x[A-Fa-f0-9]+\) must be "
                r"greater than the start address \(0x[A-Fa-f0-9]+\)"
            ],
            error=True,
        )

        # Can't read from a region without tagging
        self.expect(
            "memory tag read non_mte_buf",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+10 is not "
                "in a memory tagged region"
            ],
            error=True,
        )

        # If there's no end address we assume 1 granule
        self.expect(
            "memory tag read mte_buf",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\): 0x0 \(mismatch\)$"
            ],
        )

        # Range of <1 granule is rounded up to 1 granule
        self.expect(
            "memory tag read mte_buf mte_buf+8",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\): 0x0 \(mismatch\)$"
            ],
        )

        # Start address is aligned down, end aligned up
        self.expect(
            "memory tag read mte_buf+8 mte_buf+24",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x0 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0x1 \\(mismatch\\)$"
            ],
        )

        # You may read up to the end of the tagged region
        # Layout is mte_buf, mte_buf_2, non_mte_buf.
        # So we read from the end of mte_buf_2 here.
        self.expect(
            "memory tag read mte_buf_2+page_size-16 mte_buf_2+page_size",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+, 0x[0-9A-Fa-f]+\): 0x0$"
            ],
        )

        # Ranges with any part outside the region will error
        self.expect(
            "memory tag read mte_buf_2+page_size-16 mte_buf_2+page_size+32",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+f0:0x[0-9A-Fa-f]+20 "
                "is not in a memory tagged region"
            ],
            error=True,
        )
        self.expect(
            "memory tag read mte_buf_2+page_size",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+10 "
                "is not in a memory tagged region"
            ],
            error=True,
        )

        # You can read a range that spans more than one mapping
        # This spills into mte_buf2 which is also MTE
        self.expect(
            "memory tag read mte_buf+page_size-16 mte_buf+page_size+16",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+f0, 0x[0-9A-Fa-f]+00\\): 0xf \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x0 \\(mismatch\\)$"
            ],
        )

        # Top byte is ignored when creating the range, not just the 4 tag bits.
        # So even though these two pointers have different top bytes
        # and the start's is > the end's, this is not an error.
        self.expect(
            "memory tag read mte_buf_alt_tag mte_buf+16",
            patterns=[
                "Logical tag: 0xa\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\): 0x0 \(mismatch\)$"
            ],
        )

        # Mismatched tags are marked. The logical tag is taken from the start address.
        self.expect(
            "memory tag read mte_buf+(8*16) mte_buf_alt_tag+(8*16)+48",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+80, 0x[0-9A-Fa-f]+90\\): 0x8 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+90, 0x[0-9A-Fa-f]+a0\\): 0x9\n"
                "\\[0x[0-9A-Fa-f]+a0, 0x[0-9A-Fa-f]+b0\\): 0xa \\(mismatch\\)$"
            ],
        )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_tag_write(self):
        self.setup_mte_test()

        # Argument validation
        self.expect(
            "memory tag write",
            substrs=[
                " wrong number of arguments; expected "
                "<address-expression> <tag> [<tag> [...]]"
            ],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf",
            substrs=[
                " wrong number of arguments; expected "
                "<address-expression> <tag> [<tag> [...]]"
            ],
            error=True,
        )
        self.expect(
            "memory tag write not_a_symbol 9",
            substrs=[
                'error: Invalid address expression, address expression "not_a_symbol" '
                "evaluation failed"
            ],
            error=True,
        )

        # Can't write to a region without tagging
        self.expect(
            "memory tag write non_mte_buf 9",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+10 is not "
                "in a memory tagged region"
            ],
            error=True,
        )

        # Start address is aligned down so we write to the granule that contains it
        self.expect("memory tag write mte_buf+8 9")
        # Make sure we only modified the first granule
        self.expect(
            "memory tag read mte_buf mte_buf+32",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x9\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0x1 \\(mismatch\\)$"
            ],
        )

        # You can write multiple tags, range calculated for you
        self.expect("memory tag write mte_buf 10 11 12")
        self.expect(
            "memory tag read mte_buf mte_buf+48",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0xa \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0xb \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+20, 0x[0-9A-Fa-f]+30\\): 0xc \\(mismatch\\)$"
            ],
        )

        # You may write up to the end of a tagged region
        # (mte_buf_2's initial tags will all be 0)
        self.expect("memory tag write mte_buf_2+page_size-16 0xe")
        self.expect(
            "memory tag read mte_buf_2+page_size-16 mte_buf_2+page_size",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+, 0x[0-9A-Fa-f]+\): 0xe \(mismatch\)$"
            ],
        )

        # Ranges with any part outside the region will error
        self.expect(
            "memory tag write mte_buf_2+page_size-16 6 7",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+f0:0x[0-9A-Fa-f]+10 "
                "is not in a memory tagged region"
            ],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf_2+page_size 6",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+10 "
                "is not in a memory tagged region"
            ],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf_2+page_size 6 7 8",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+30 "
                "is not in a memory tagged region"
            ],
            error=True,
        )

        # You can write to a range that spans two mappings, as long
        # as they are both tagged.
        # buf and buf2 are next to each other so this wirtes into buf2.
        self.expect("memory tag write mte_buf+page_size-16 1 2")
        self.expect(
            "memory tag read mte_buf+page_size-16 mte_buf+page_size+16",
            patterns=[
                "Logical tag: 0x9\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+f0, 0x[0-9A-Fa-f]+00\\): 0x1 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x2 \\(mismatch\\)$"
            ],
        )

        # Even if a page is read only the debugger can still write to it
        self.expect("memory tag write mte_read_only 1")
        self.expect(
            "memory tag read mte_read_only",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                r"\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\): 0x1 \(mismatch\)$"
            ],
        )

        # Trying to write a value > maximum tag value is an error
        self.expect(
            "memory tag write mte_buf 99",
            patterns=["error: Found tag 0x63 which is > max MTE tag value of 0xf."],
            error=True,
        )

        # You can provide an end address and have lldb repeat the tags as needed
        # The range is checked in the same way it is for "memory tag read"
        self.expect(
            "memory tag write mte_buf 9 -e",
            patterns=["error: last option requires an argument"],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf 9 -e food",
            patterns=['error: address expression "food" evaluation failed'],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf_2 9 --end-addr mte_buf_2",
            patterns=[
                r"error: End address \(0x[A-Fa-f0-9]+\) must be "
                r"greater than the start address \(0x[A-Fa-f0-9]+\)"
            ],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf_2 9 --end-addr mte_buf_2-16",
            patterns=[
                r"error: End address \(0x[A-Fa-f0-9]+\) must be "
                r"greater than the start address \(0x[A-Fa-f0-9]+\)"
            ],
            error=True,
        )
        self.expect(
            "memory tag write mte_buf_2 9 --end-addr mte_buf_2+page_size+16",
            patterns=[
                "error: Address range 0x[0-9A-Fa-f]+00:0x[0-9A-Fa-f]+10 "
                "is not in a memory tagged region"
            ],
            error=True,
        )

        # Tags are repeated across the range
        # For these we'll read one extra to make sure we don't over write
        self.expect("memory tag write mte_buf_2 4 5 --end-addr mte_buf_2+48")
        self.expect(
            "memory tag read mte_buf_2 mte_buf_2+64",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x4 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0x5 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+20, 0x[0-9A-Fa-f]+30\\): 0x4 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+30, 0x[0-9A-Fa-f]+40\\): 0x0$"
            ],
        )

        # Since this aligns like tag read does, the start is aligned down and the end up.
        # Meaning that start/end tells you the start/end granule that will be written.
        # This matters particularly if either are misaligned.

        # Here start moves down so the final range is mte_buf_2 -> mte_buf_2+32
        self.expect("memory tag write mte_buf_2+8 6 -end-addr mte_buf_2+32")
        self.expect(
            "memory tag read mte_buf_2 mte_buf_2+48",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x6 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0x6 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+20, 0x[0-9A-Fa-f]+30\\): 0x4 \\(mismatch\\)$"
            ],
        )

        # If we do the same with a misaligned end, it also moves but upward.
        # The initial range is 2 granules but the final range is mte_buf_2 -> mte_buf_2+48
        self.expect("memory tag write mte_buf_2+8 3 -end-addr mte_buf_2+32+8")
        self.expect(
            "memory tag read mte_buf_2 mte_buf_2+64",
            patterns=[
                "Logical tag: 0x0\n"
                "Allocation tags:\n"
                "\\[0x[0-9A-Fa-f]+00, 0x[0-9A-Fa-f]+10\\): 0x3 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+10, 0x[0-9A-Fa-f]+20\\): 0x3 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+20, 0x[0-9A-Fa-f]+30\\): 0x3 \\(mismatch\\)\n"
                "\\[0x[0-9A-Fa-f]+30, 0x[0-9A-Fa-f]+40\\): 0x0$"
            ],
        )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_memory_read_tag_display(self):
        self.setup_mte_test()

        # Reading from an untagged range should not be any different
        self.expect(
            "memory read non_mte_buf non_mte_buf+16", substrs=["tag"], matching=False
        )

        # show-tags option is required
        self.expect(
            'memory read mte_buf mte_buf+32 -f "x" -l 1 -s 16',
            patterns=["tag"],
            matching=False,
        )

        # Reading 16 bytes per line means 1 granule and so 1 tag per line
        self.expect(
            'memory read mte_buf mte_buf+32 -f "x" -l 1 -s 16 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+00: 0x0+ \\(tag: 0x0\\)\n"
                "0x[0-9A-Fa-f]+10: 0x0+ \\(tag: 0x1\\)"
            ],
        )

        # If bytes per line is > granule size then you get multiple tags
        # per line.
        self.expect(
            'memory read mte_buf mte_buf+32 -f "x" -l 1 -s 32 --show-tags',
            patterns=["0x[0-9A-Fa-f]+00: 0x0+ \\(tags: 0x0 0x1\\)\n"],
        )

        # Reading half a granule still shows you the tag for that granule
        self.expect(
            'memory read mte_buf mte_buf+8 -f "x" -l 1 -s 8 --show-tags',
            patterns=["0x[0-9A-Fa-f]+00: 0x0+ \\(tag: 0x0\\)\n"],
        )

        # We can read a whole number of granules but split them over more lines
        # than there are granules. Tags are shown repeated for each applicable line.
        self.expect(
            'memory read mte_buf+32 mte_buf+64 -f "x" -l 1 -s 8 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+20: 0x0+ \\(tag: 0x2\\)\n"
                "0x[0-9A-Fa-f]+28: 0x0+ \\(tag: 0x2\\)\n"
                "0x[0-9A-Fa-f]+30: 0x0+ \\(tag: 0x3\\)\n"
                "0x[0-9A-Fa-f]+38: 0x0+ \\(tag: 0x3\\)"
            ],
        )

        # Also works if we misalign the start address. Note the first tag is shown
        # only once here and we have a new tag on the last line.
        # (bytes per line == the misalignment here)
        self.expect(
            'memory read mte_buf+32+8 mte_buf+64+8 -f "x" -l 1 -s 8 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+28: 0x0+ \\(tag: 0x2\\)\n"
                "0x[0-9A-Fa-f]+30: 0x0+ \\(tag: 0x3\\)\n"
                "0x[0-9A-Fa-f]+38: 0x0+ \\(tag: 0x3\\)\n"
                "0x[0-9A-Fa-f]+40: 0x0+ \\(tag: 0x4\\)"
            ],
        )

        # We can do the same thing but where the misaligment isn't equal to
        # bytes per line. This time, some lines cover multiple granules and
        # so show multiple tags.
        self.expect(
            'memory read mte_buf+32+4 mte_buf+64+4 -f "x" -l 1 -s 8 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+24: 0x0+ \\(tag: 0x2\\)\n"
                "0x[0-9A-Fa-f]+2c: 0x0+ \\(tags: 0x2 0x3\\)\n"
                "0x[0-9A-Fa-f]+34: 0x0+ \\(tag: 0x3\\)\n"
                "0x[0-9A-Fa-f]+3c: 0x0+ \\(tags: 0x3 0x4\\)"
            ],
        )

        # If you read a range that includes non tagged areas those areas
        # simply aren't annotated.

        # Initial part of range is untagged
        self.expect(
            'memory read mte_buf-16 mte_buf+32 -f "x" -l 1 -s 16 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+f0: 0x0+\n"
                "0x[0-9A-Fa-f]+00: 0x0+ \\(tag: 0x0\\)\n"
                "0x[0-9A-Fa-f]+10: 0x0+ \\(tag: 0x1\\)"
            ],
        )

        # End of range is untagged
        self.expect(
            'memory read mte_buf+page_size-16 mte_buf+page_size+16 -f "x" -l 1 -s 16 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+f0: 0x0+ \\(tag: 0xf\\)\n" "0x[0-9A-Fa-f]+00: 0x0+"
            ],
        )

        # The smallest MTE range we can get is a single page so we just check
        # parts of this result. Where we read from before the tagged page to after it.
        # Add --force here because we're reading just over 4k.
        self.expect(
            'memory read mte_read_only-16 mte_read_only+page_size+16 -f "x" -l 1 -s 16 --force --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+f0: 0x0+\n" "0x[0-9A-Fa-f]+00: 0x0+ \\(tag: 0x0\\)\n",
                "0x[0-9A-Fa-f]+f0: 0x0+ \\(tag: 0x0\\)\n" "0x[0-9A-Fa-f]+00: 0x0+",
            ],
        )

        # Some parts of a line might be tagged and others untagged.
        # <no tag> is shown in where the tag would be, to keep the order intact.
        self.expect(
            'memory read mte_buf-16 mte_buf+32 -f "x" -l 1 -s 32 --show-tags',
            patterns=[r"0x[0-9A-Fa-f]+f0: 0x0+ \(tags: <no tag> 0x0\)"],
        )
        self.expect(
            'memory read mte_read_only+page_size-16 mte_read_only+page_size+16 -f "x" -l 1 -s 32  --show-tags',
            patterns=[r"0x[0-9A-Fa-f]+f0: 0x0+ \(tags: 0x0 <no tag>\)"],
        )

        # Here the start address is unaligned so we cover 3 granules instead of 2
        self.expect(
            'memory read mte_buf-16+4 mte_buf+32+4 -f "x" -l 1 -s 32 --show-tags',
            patterns=[r"0x[0-9A-Fa-f]+f4: 0x0+ \(tags: <no tag> 0x0 0x1\)"],
        )
        self.expect(
            'memory read mte_read_only+page_size-16+4 mte_read_only+page_size+16+4 -f "x" -l 1 -s 32 --show-tags',
            patterns=[r"0x[0-9A-Fa-f]+f4: 0x0+ \(tags: 0x0 <no tag> <no tag>\)"],
        )

        # Some formats call DumpDataExtractor multiple times,
        # check that those print tags only once per line.
        self.expect(
            'memory read mte_buf mte_buf+32 -f "x" --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+00: 0x0+ 0x0+ 0x0+ 0x0+ \\(tag: 0x0\\)\n",
                "0x[0-9A-Fa-f]+10: 0x0+ 0x0+ 0x0+ 0x0+ \\(tag: 0x1\\)",
            ],
        )

        self.expect(
            'memory read mte_buf mte_buf+32 -f "bytes with ASCII" --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+00: (00 ){16} \\.{16} \\(tag: 0x0\\)\n",
                "0x[0-9A-Fa-f]+10: (00 ){16} \\.{16} \\(tag: 0x1\\)",
            ],
        )

        self.expect(
            'memory read mte_buf mte_buf+32 -f "uint8_t[]" -s 16 -l 1 --show-tags',
            patterns=[
                "0x[0-9A-Fa-f]+00: \\{(0x00 ){15}0x00\\} \\(tag: 0x0\\)\n"
                "0x[0-9A-Fa-f]+10: \\{(0x00 ){15}0x00\\} \\(tag: 0x1\\)"
            ],
        )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_memory_read_tag_display_repeated(self):
        """Test that the --show-tags option is kept when repeating the memory read command."""
        self.setup_mte_test()

        self.expect(
            'memory read mte_buf mte_buf+16 -f "x" --show-tags',
            patterns=[r"0x[0-9A-fa-f]+00: 0x0+ 0x0+ 0x0+ 0x0+ \(tag: 0x0\)"],
        )
        # Equivalent to just pressing enter on the command line.
        self.expect(
            "memory read",
            patterns=[r"0x[0-9A-fa-f]+10: 0x0+ 0x0+ 0x0+ 0x0+ \(tag: 0x1\)"],
        )

        # You can add the argument to an existing repetition without resetting
        # the whole command. Though all other optional arguments will reset to
        # their default values when you do this.
        self.expect(
            'memory read mte_buf mte_buf+16 -f "x"',
            patterns=["0x[0-9A-fa-f]+00: 0x0+ 0x0+ 0x0+ 0x0+"],
        )
        self.expect("memory read", patterns=["0x[0-9A-fa-f]+10: 0x0+ 0x0+ 0x0+ 0x0+"])
        # Note that the formatting returns to default here.
        self.expect(
            "memory read --show-tags",
            patterns=[r"0x[0-9A-fa-f]+20: (00 )+ \.+ \(tag: 0x2\)"],
        )
        self.expect(
            "memory read", patterns=[r"0x[0-9A-fa-f]+30: (00 )+ \.+ \(tag: 0x3\)"]
        )

        # A fresh command reverts to the default of tags being off.
        self.expect(
            'memory read mte_buf mte_buf+16 -f "x"',
            patterns=["0x[0-9A-fa-f]+00: 0x0+ 0x0+ 0x0+ 0x0+"],
        )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_memory_find(self):
        """Test the --show-tags option with memory find."""
        self.setup_mte_test()

        # No result, nothing changes.
        self.expect(
            'memory find -s "foo" mte_buf mte_buf+32 --show-tags',
            substrs=["data not found within the range."],
        )

        cmd = 'memory find -s "LLDB" mte_buf+64 mte_buf+512'
        found_pattern = "data found at location: 0x[0-9A-Fa-f]+80"
        results_patterns = [
            r"0x[0-9A-Fa-f]+80: 4c 4c 44 42 (00 )+ LLDB\.+",
            r"0x[0-9A-Fa-f]+90: 00 00 00 00 (00 )+ \.+",
        ]

        # Default is not to show tags
        self.expect(cmd, patterns=[found_pattern, *results_patterns])
        self.expect(
            cmd + " --show-tags",
            patterns=[
                found_pattern,
                results_patterns[0] + r" \(tag: 0x8\)",
                results_patterns[1] + r" \(tag: 0x9\)",
            ],
        )

        # Uses the same logic as memory read to handle misalignment.
        self.expect(
            'memory find -s "DB" mte_buf+64 mte_buf+512 --show-tags',
            patterns=[
                "data found at location: 0x[0-9A-Fa-f]+82\n"
                "0x[0-9A-Fa-f]+82: 44 42 (00 )+ DB\\.+ \\(tags: 0x8 0x9\\)\n",
                "0x[0-9A-Fa-f]+92: 00 00 (00 )+ ..\\.+ \\(tags: 0x9 0xa\\)",
            ],
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg

from mlir.dialects.linalg.opdsl.lang import *

# This tests miscellaneous features of the emitter that are not tested by the
# fill, matmul, convolution, or pooling tests. The features include:
# - constant defined in the body
# - fix/predefined types
# - some math/arith functions, including abs, ceil, exp, floor, log, and negf
# - custom op names.


@linalg_structured_op
def test_const(O=TensorDef(F32, S.M, S.N, output=True)):
    O[D.m, D.n] = TypeFn.cast_unsigned(F32, const(42)) + TypeFn.cast_unsigned(
        F32, const(2.3283064e-10)
    )


@linalg_structured_op
def test_index(O=TensorDef(I32, S.M, S.N, output=True)):
    O[D.m, D.n] = TypeFn.cast_signed(I32, index(D.m)) + TypeFn.cast_signed(
        I32, index(D.n)
    )


@linalg_structured_op
def elemwise_unary_poly(
    I=TensorDef(T),
    O=TensorDef(U, output=True),
    fun=UnaryFnAttrDef(default=UnaryFn.exp),
    cast=TypeFnAttrDef(default=TypeFn.cast_signed),
):
    O[None] = fun(cast(U, I[None]))


@linalg_structured_op(op_name="custom_op_name")
def non_default_op_name(I=TensorDef(T, S.N), O=TensorDef(T, S.N, output=True)):
    O[D.n] = I[D.n]


with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    c32 = ComplexType.get(f32)
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

        # CHECK-LABEL: @test_f32_const
        # CHECK-DAG:    %[[CST0:.+]] = arith.constant 42 : i64
        # CHECK-DAG:    %[[CST0_CAST:.+]] = arith.uitofp %[[CST0]] : i64 to f32
        # CHECK-DAG:    %[[CST1:.+]] = arith.constant 2.3283063999999999E-10 : f64
        # CHECK-DAG:    %[[CST1_CAST:.+]] = arith.truncf %[[CST1]] : f64 to f32
        # CHECK-DAG:    %[[SUM:.+]] = arith.addf %[[CST0_CAST]], %[[CST1_CAST]] : f32
        # CHECK-NEXT:   linalg.yield %[[SUM]] : f32
        @func.FuncOp.from_py_func(RankedTensorType.get((4, 16), f32))
        def test_f32_const(init_result):
            return test_const(outs=[init_result])

        # CHECK-LABEL: @test_i32_index
        # CHECK-DAG:    %[[IDX0:.+]] = linalg.index 0 : index
        # CHECK-DAG:    %[[IDX1:.+]] = linalg.index 1 : index
        # CHECK-DAG:    %[[IDX0_CAST:.+]] = arith.index_cast %[[IDX0]] : index to i32
        # CHECK-DAG:    %[[IDX1_CAST:.+]] = arith.index_cast %[[IDX1]] : index to i32
        # CHECK-DAG:    %[[SUM:.+]] = arith.addi %[[IDX0_CAST]], %[[IDX1_CAST]] : i32
        # CHECK-NEXT:   linalg.yield %[[SUM]] : i32
        @func.FuncOp.from_py_func(RankedTensorType.get((4, 16), i32))
        def test_i32_index(init_result):
            return test_index(outs=[init_result])

        # CHECK-LABEL: @test_f32_elemwise_exp
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[EXP:.+]] = math.exp %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[EXP]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_exp(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.exp)

        # CHECK-LABEL: @test_f32_elemwise_log
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[LOG:.+]] = math.log %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[LOG]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_log(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.log)

        # CHECK-LABEL: @test_f32_elemwise_abs
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[EXP:.+]] = math.absf %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[EXP]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_abs(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.abs)

        # CHECK-LABEL: @test_f32_elemwise_ceil
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[EXP:.+]] = math.ceil %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[EXP]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_ceil(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.ceil)

        # CHECK-LABEL: @test_f32_elemwise_floor
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[EXP:.+]] = math.floor %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[EXP]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_floor(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.floor)

        # CHECK-LABEL: @test_f32_elemwise_neg
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[EXP:.+]] = arith.negf %[[IN]] : f32
        # CHECK-NEXT:   linalg.yield %[[EXP]] : f32
        # CHECK-NEXT: -> tensor<4x16xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((4, 16), f32)
        )
        def test_f32_elemwise_neg(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.negf)

        # CHECK-LABEL: @test_c32_elemwise_neg
        # CHECK:      ^{{.*}}(%[[IN:.+]]: complex<f32>, %[[OUT:.+]]: complex<f32>)
        # CHECK-NEXT:   %[[EXP:.+]] = complex.neg %[[IN]] : complex<f32>
        # CHECK-NEXT:   linalg.yield %[[EXP]] : complex<f32>
        # CHECK-NEXT: -> tensor<4x16xcomplex<f32>>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), c32), RankedTensorType.get((4, 16), c32)
        )
        def test_c32_elemwise_neg(input, init_result):
            return elemwise_unary_poly(input, outs=[init_result], fun=UnaryFn.negf)

        # Just check that we don't assert out on name mismatch.
        # CHECK-LABEL: @test_non_default_op_name
        @func.FuncOp.from_py_func(
            RankedTensorType.get((42,), f32), RankedTensorType.get((42,), f32)
        )
        def test_non_default_op_name(input, init_result):
            return non_default_op_name(input, outs=[init_result])


print(module)



"""
Test that lldb removes non-address bits in situations where they would cause
failures if not removed. Like when reading memory. Tests are done at command
and API level because commands may remove non-address bits for display
reasons which can make it seem like the operation as a whole works but at the
API level it won't if we don't remove them there also.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxNonAddressBitMemoryAccessTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setup_test(self):
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    def check_cmd_read_write(self, write_to, read_from, data):
        self.runCmd("memory write {} {}".format(write_to, data))
        self.expect("memory read {}".format(read_from), substrs=[data])

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_non_address_bit_memory_read_write_cmds(self):
        self.setup_test()

        # Writes should be visible through either pointer
        self.check_cmd_read_write("buf", "buf", "01 02 03 04")
        self.check_cmd_read_write(
            "buf_with_non_address", "buf_with_non_address", "02 03 04 05"
        )
        self.check_cmd_read_write("buf", "buf_with_non_address", "03 04 05 06")
        self.check_cmd_read_write("buf_with_non_address", "buf", "04 05 06 07")

        # Printing either should get the same result
        self.expect("expression -f hex -- *(uint32_t*)buf", substrs=["0x07060504"])
        self.expect(
            "expression -f hex -- *(uint32_t*)buf_with_non_address",
            substrs=["0x07060504"],
        )

    def get_ptr_values(self):
        frame = self.process().GetThreadAtIndex(0).GetFrameAtIndex(0)
        buf = frame.FindVariable("buf").GetValueAsUnsigned()
        buf_with_non_address = frame.FindVariable(
            "buf_with_non_address"
        ).GetValueAsUnsigned()
        return buf, buf_with_non_address

    def check_api_read_write(self, write_to, read_from, data):
        error = lldb.SBError()
        written = self.process().WriteMemory(write_to, data, error)
        self.assertTrue(error.Success())
        self.assertEqual(len(data), written)
        buf_content = self.process().ReadMemory(read_from, 4, error)
        self.assertTrue(error.Success())
        self.assertEqual(data, buf_content)

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_non_address_bit_memory_read_write_api_process(self):
        self.setup_test()
        buf, buf_with_non_address = self.get_ptr_values()

        # Writes are visible through either pointer
        self.check_api_read_write(buf, buf, bytes([0, 1, 2, 3]))
        self.check_api_read_write(
            buf_with_non_address, buf_with_non_address, bytes([1, 2, 3, 4])
        )
        self.check_api_read_write(buf, buf_with_non_address, bytes([2, 3, 4, 5]))
        self.check_api_read_write(buf_with_non_address, buf, bytes([3, 4, 5, 6]))

        # Now check all the "Read<type>FromMemory" don't fail
        error = lldb.SBError()
        # Last 4 bytes are just for the pointer read
        data = bytes([0x4C, 0x4C, 0x44, 0x42, 0x00, 0x12, 0x34, 0x56])
        written = self.process().WriteMemory(buf, data, error)
        self.assertTrue(error.Success())
        self.assertEqual(len(data), written)

        # C string
        c_string = self.process().ReadCStringFromMemory(buf_with_non_address, 5, error)
        self.assertTrue(error.Success())
        self.assertEqual("LLDB", c_string)

        # Unsigned
        unsigned_num = self.process().ReadUnsignedFromMemory(
            buf_with_non_address, 4, error
        )
        self.assertTrue(error.Success())
        self.assertEqual(0x42444C4C, unsigned_num)

        # Pointer
        ptr = self.process().ReadPointerFromMemory(buf_with_non_address, error)
        self.assertTrue(error.Success())
        self.assertEqual(0x5634120042444C4C, ptr)

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_non_address_bit_memory_read_write_api_target(self):
        self.setup_test()
        buf, buf_with_non_address = self.get_ptr_values()

        # Target only has ReadMemory
        error = lldb.SBError()
        data = bytes([1, 2, 3, 4])
        written = self.process().WriteMemory(buf, data, error)
        self.assertTrue(error.Success())
        self.assertEqual(len(data), written)

        addr = lldb.SBAddress()
        addr.SetLoadAddress(buf, self.target())
        buf_read = self.target().ReadMemory(addr, 4, error)
        self.assertTrue(error.Success())
        self.assertEqual(data, buf_read)

        addr.SetLoadAddress(buf_with_non_address, self.target())
        buf_non_address_read = self.target().ReadMemory(addr, 4, error)
        self.assertTrue(error.Success())
        self.assertEqual(data, buf_non_address_read)

        # Read<type>FromMemory are in Target but not SBTarget so no tests for those.

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_non_address_bit_memory_caching(self):
        # The read/write tests above do exercise the cache but this test
        # only cares that the cache sees buf and buf_with_non_address
        # as the same location.
        self.setup_test()
        buf, buf_with_non_address = self.get_ptr_values()

        # Enable packet logging so we can see when reads actually
        # happen.
        log_file = self.getBuildArtifact("lldb-non-address-bit-log.txt")
        # This defaults to overwriting the file so we don't need to delete
        # any existing files.
        self.runCmd("log enable gdb-remote packets -f '%s'" % log_file)

        # This should fill the cache by doing a read of buf_with_non_address
        # with the non-address bits removed (which is == buf).
        self.runCmd("expression buf_with_non_address")
        # This will read from the cache since the two pointers point to the
        # same place.
        self.runCmd("expression buf")

        # Open log ignoring utf-8 decode errors
        with open(log_file, "r", errors="ignore") as f:
            read_packet = "send packet: $x{:x}"

            # Since we allocated a 4k page that page will be aligned to 4k, which
            # also fits the 512 byte line size of the memory cache. So we can expect
            # to find a packet with its exact address.
            read_buf_packet = read_packet.format(buf)
            read_buf_with_non_address_packet = read_packet.format(buf_with_non_address)

            # We expect to find 1 and only 1 read of buf.
            # We expect to find no reads using buf_with_no_address.
            found_read_buf = False
            for line in f:
                if read_buf_packet in line:
                    if found_read_buf:
                        self.fail("Expected 1 read of buf but found more than one.")
                    found_read_buf = True

                if read_buf_with_non_address_packet in line:
                    self.fail("Unexpected read of buf_with_non_address found.")

            if not found_read_buf:
                self.fail("Did not find any reads of buf.")

    @skipIfLLVMTargetMissing("AArch64")
    def test_non_address_bit_memory_corefile(self):
        self.runCmd("target create --core corefile")

        self.expect(
            "thread list",
            substrs=[
                "stopped",
                "stop reason = SIGSEGV: address not mapped to object (fault address=0x0)",
            ],
        )

        # No caching (the program/corefile are the cache) and no writing
        # to memory. So just check that tagged/untagged addresses read
        # the same location.

        # These are known addresses in the corefile, since we won't have symbols.
        buf = 0x0000FFFFA75A5000
        buf_with_non_address = 0xFF0BFFFFA75A5000

        expected = ["4c 4c 44 42", "LLDB"]
        self.expect("memory read 0x{:x}".format(buf), substrs=expected)
        self.expect("memory read 0x{:x}".format(buf_with_non_address), substrs=expected)

        # This takes a more direct route to ReadMemory. As opposed to "memory read"
        # above that might fix the addresses up front for display reasons.
        self.expect("expression (char*)0x{:x}".format(buf), substrs=["LLDB"])
        self.expect(
            "expression (char*)0x{:x}".format(buf_with_non_address), substrs=["LLDB"]
        )

        def check_reads(addrs, method, num_bytes, expected):
            error = lldb.SBError()
            for addr in addrs:
                if num_bytes is None:
                    got = method(addr, error)
                else:
                    got = method(addr, num_bytes, error)

            self.assertTrue(error.Success())
            self.assertEqual(expected, got)

        addr_buf = lldb.SBAddress()
        addr_buf.SetLoadAddress(buf, self.target())
        addr_buf_with_non_address = lldb.SBAddress()
        addr_buf_with_non_address.SetLoadAddress(buf_with_non_address, self.target())
        check_reads(
            [addr_buf, addr_buf_with_non_address], self.target().ReadMemory, 4, b"LLDB"
        )

        addrs = [buf, buf_with_non_address]
        check_reads(addrs, self.process().ReadMemory, 4, b"LLDB")
        check_reads(addrs, self.process().ReadCStringFromMemory, 5, "LLDB")
        check_reads(addrs, self.process().ReadUnsignedFromMemory, 4, 0x42444C4C)
        check_reads(
            addrs, self.process().ReadPointerFromMemory, None, 0x0000000042444C4C
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        # First run this in C which should work.
        self.expect_expr("constexpr.class", result_type="int", result_value="3")

        # Now try running this in a language that explicitly enables C++.
        # This isn't expected to work, but at least it shouldn't crash LLDB.
        self.expect(
            "expr -l c++ -- constexpr.class",
            error=True,
            substrs=["expected unqualified-id"],
        )
        self.expect(
            "expr -l objective-c++ -- constexpr.class",
            error=True,
            substrs=["expected unqualified-id"],
        )



"""
Test lookup unnamed symbols.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

# --keep-symbol causes error on Windows: llvm-strip.exe: error: option is not supported for COFF
@skipIfWindows
# Unnamed symbols don't get into the .eh_frame section on ARM, so LLDB can't find them.
@skipIf(archs=["arm$"])
class TestUnnamedSymbolLookup(TestBase):
    def test_unnamed_symbol_lookup(self):
        """Test looking up unnamed symbol synthetic name"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(
            self, "main", exe_name="a.out.stripped"
        )

        main_frame = thread.GetFrameAtIndex(0)

        # Step until reaching the unnamed symbol called from main
        for _ in range(100):
            thread.StepInto()
            if thread.GetFrameAtIndex(0) != main_frame:
                break

            thread.StepInto()

        self.assertEqual(
            main_frame, thread.GetFrameAtIndex(1), "Expected to be called from main"
        )
        symbol = thread.GetFrameAtIndex(0).GetSymbol()
        self.assertIsNotNone(symbol, "unnamed symbol called from main not reached")
        self.assertTrue(symbol.name.startswith("___lldb_unnamed_symbol"))

        exe_module = symbol.GetStartAddress().GetModule()
        found_symbols = exe_module.FindSymbols(symbol.name)
        self.assertIsNotNone(found_symbols)
        self.assertEqual(found_symbols.GetSize(), 1)



""" Check that when a debug server provides XML that only defines SVE Z registers,
    and does not include Neon V registers, lldb creates sub-registers to represent
    the V registers as the bottom 128 bits of the Z registers.

    qemu-aarch64 is one such debug server.

    This also doubles as a test that lldb has a fallback path for registers of
    unknown type that are > 128 bits, as the SVE registers are here.
"""

from textwrap import dedent
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class Responder(MockGDBServerResponder):
    def __init__(self):
        super().__init__()
        self.vg = 4
        self.pc = 0xA0A0A0A0A0A0A0A0

    def qXferRead(self, obj, annex, offset, length):
        if annex == "target.xml":
            # Note that QEMU sends the current SVE size in XML and the debugger
            # then reads vg to know the latest size.
            return (
                dedent(
                    """\
              <?xml version="1.0"?>
              <target version="1.0">
                <architecture>aarch64</architecture>
                <feature name="org.gnu.gdb.aarch64.core">
                  <reg name="pc" regnum="0" bitsize="64"/>
                  <reg name="vg" regnum="1" bitsize="64"/>
                  <reg name="z0" regnum="2" bitsize="2048" type="not_a_type"/>
                </feature>
              </target>"""
                ),
                False,
            )

        return (None,)

    def readRegister(self, regnum):
        return "E01"

    def readRegisters(self):
        return "".join(
            [
                # 64 bit PC.
                f"{self.pc:x}",
                # 64 bit vg
                f"0{self.vg}00000000000000",
                # Enough data for 256 and 512 bit SVE.
                "".join([f"{n:02x}" * 4 for n in range(1, 17)]),
            ]
        )

    def cont(self):
        # vg is expedited so that lldb can resize the SVE registers.
        return f"T02thread:1ff0d;threads:1ff0d;thread-pcs:{self.pc};01:0{self.vg}00000000000000;"

    def writeRegisters(self, registers_hex):
        # We get a block of data containing values in regnum order.
        self.vg = int(registers_hex[16:18])
        return "OK"


class TestXMLRegisterFlags(GDBRemoteTestBase):
    def check_regs(self, vg):
        # Each 32 bit chunk repeats n.
        z0_value = " ".join(
            [" ".join([f"0x{n:02x}"] * 4) for n in range(1, (vg * 2) + 1)]
        )

        self.expect(
            "register read vg z0 v0 s0 d0",
            substrs=[
                f"      vg = 0x000000000000000{vg}\n"
                "      z0 = {" + z0_value + "}\n"
                "      v0 = {0x01 0x01 0x01 0x01 0x02 0x02 0x02 0x02 0x03 0x03 0x03 0x03 0x04 0x04 0x04 0x04}\n"
                "      s0 = 2.36942783E-38\n"
                "      d0 = 5.3779407333977203E-299\n"
            ],
        )

        self.expect("register read s0 --format uint32", substrs=["s0 = {0x01010101}"])
        self.expect(
            "register read d0 --format uint64",
            substrs=["d0 = {0x0202020201010101}"],
        )

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("AArch64")
    def test_v_sub_registers(self):
        self.server.responder = Responder()
        target = self.dbg.CreateTarget("")

        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.check_regs(4)

        # Now increase the SVE length and continue. The mock will respond with a new
        # vg and lldb will reconfigure the register defs. This should not break the
        # sub-registers.

        self.runCmd("register write vg 8")
        self.expect("continue", substrs=["stop reason = signal SIGINT"])

        self.check_regs(8)



"""
Test lldb's handling of job control signals (SIGTSTP, SIGCONT).
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class JobControlTest(PExpectTest):
    @skipIf(macos_version=["<", "14.0"], asan=True)
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])
    def test_job_control(self):
        def post_spawn():
            self.child.expect("PID=([0-9]+)")
            self.lldb_pid = int(self.child.match[1])

        run_under = [sys.executable, self.getSourcePath("shell.py")]
        self.launch(run_under=run_under, post_spawn=post_spawn)

        os.kill(self.lldb_pid, signal.SIGTSTP)
        self.child.expect("STATUS=([0-9]+)")
        status = int(self.child.match[1])

        self.assertTrue(os.WIFSTOPPED(status))
        self.assertEqual(os.WSTOPSIG(status), signal.SIGTSTP)

        os.kill(self.lldb_pid, signal.SIGCONT)

        self.child.sendline("quit")
        self.child.expect("RETURNCODE=0")



#!/usr/bin/env python3

"""
This script reads the input from stdin, extracts all lines starting with
"# FDATA: " (or a given prefix instead of "FDATA"), parses the directives,
replaces symbol names ("#name#") with either symbol values or with offsets from
respective anchor symbols, and prints the resulting file to stdout.
"""

import argparse
import os
import platform
import shutil
import subprocess
import sys
import re

parser = argparse.ArgumentParser()
parser.add_argument("input")
parser.add_argument("objfile", help="Object file to extract symbol values from")
parser.add_argument("output")
parser.add_argument("prefix", nargs="?", default="FDATA", help="Custom FDATA prefix")
parser.add_argument(
    "--nmtool",
    default="llvm-nm" if platform.system() == "Windows" else "nm",
    help="Path to nm tool",
)
parser.add_argument("--no-lbr", action="store_true")
parser.add_argument("--no-redefine", action="store_true")

args = parser.parse_args()

# Regexes to extract FDATA lines from input and parse FDATA and pre-aggregated
# profile data
prefix_pat = re.compile(f"^(#|//) {args.prefix}: (.*)")

# FDATA records:
# <is symbol?> <closest elf symbol or DSO name> <relative FROM address>
# <is symbol?> <closest elf symbol or DSO name> <relative TO address>
# <number of mispredictions> <number of branches>
fdata_pat = re.compile(r"([01].*) (?P<mispred>\d+) (?P<exec>\d+)")

# Pre-aggregated profile:
# {T|R|S|E|B|F|f|r} <start> [<end>] [<ft_end>] <count> [<mispred_count>]
# <loc>: [<id>:]<offset>
preagg_pat = re.compile(r"(?P<type>[TRSBFfr]) (?P<offsets_count>.*)")

# No-LBR profile:
# <is symbol?> <closest elf symbol or DSO name> <relative address> <count>
nolbr_pat = re.compile(r"([01].*) (?P<count>\d+)")

# Replacement symbol: #symname#
replace_pat = re.compile(r"#(?P<symname>[^#]+)#")

# Read input and construct the representation of fdata expressions
# as (src_tuple, dst_tuple, mispred_count, exec_count) tuples, where src and dst
# are represented as (is_sym, anchor, offset) tuples
exprs = []
with open(args.input, "r") as f:
    for line in f.readlines():
        prefix_match = prefix_pat.match(line)
        if not prefix_match:
            continue
        profile_line = prefix_match.group(2)
        fdata_match = fdata_pat.match(profile_line)
        preagg_match = preagg_pat.match(profile_line)
        nolbr_match = nolbr_pat.match(profile_line)
        if fdata_match:
            src_dst, mispred, execnt = fdata_match.groups()
            # Split by whitespaces not preceded by a backslash (negative lookbehind)
            chunks = re.split(r"(?<!\\) +", src_dst)
            # Check if the number of records separated by non-escaped whitespace
            # exactly matches the format.
            assert (
                len(chunks) == 6
            ), f"ERROR: wrong format/whitespaces must be escaped:\n{line}"
            exprs.append(("FDATA", (*chunks, mispred, execnt)))
        elif nolbr_match:
            loc, count = nolbr_match.groups()
            # Split by whitespaces not preceded by a backslash (negative lookbehind)
            chunks = re.split(r"(?<!\\) +", loc)
            # Check if the number of records separated by non-escaped whitespace
            # exactly matches the format.
            assert (
                len(chunks) == 3
            ), f"ERROR: wrong format/whitespaces must be escaped:\n{line}"
            exprs.append(("NOLBR", (*chunks, count)))
        elif preagg_match:
            exprs.append(("PREAGG", preagg_match.groups()))
        else:
            exit("ERROR: unexpected input:\n%s" % line)

# Read nm output: <symbol value> <symbol type> <symbol name>
# Ignore .exe on Windows host.
is_llvm_nm = os.path.basename(os.path.realpath(shutil.which(args.nmtool))).startswith(
    "llvm-nm"
)
nm_output = subprocess.run(
    [
        args.nmtool,
        "--defined-only",
        "--special-syms" if is_llvm_nm else "--synthetic",
        args.objfile,
    ],
    text=True,
    capture_output=True,
).stdout
# Populate symbol map
symbols = {}
for symline in nm_output.splitlines():
    symval, _, symname = symline.split(maxsplit=2)
    if symname in symbols and args.no_redefine:
        continue
    symbols[symname] = symval


def evaluate_symbol(issym, anchor, offsym):
    sym_match = replace_pat.match(offsym)
    if not sym_match:
        # No need to evaluate symbol value, return as is
        return f"{issym} {anchor} {offsym}"
    symname = sym_match.group("symname")
    assert symname in symbols, f"ERROR: symbol {symname} is not defined in binary"
    # Evaluate to an absolute offset if issym is false
    if issym == "0":
        return f"{issym} {anchor} {symbols[symname]}"
    # Evaluate symbol against its anchor if issym is true
    assert anchor in symbols, f"ERROR: symbol {anchor} is not defined in binary"
    anchor_value = int(symbols[anchor], 16)
    symbol_value = int(symbols[symname], 16)
    sym_offset = symbol_value - anchor_value
    return f'{issym} {anchor} {format(sym_offset, "x")}'


def replace_symbol(matchobj):
    """
    Expects matchobj to only capture one group which contains the symbol name.
    """
    symname = matchobj.group("symname")
    assert symname in symbols, f"ERROR: symbol {symname} is not defined in binary"
    return symbols[symname]


with open(args.output, "w", newline="\n") as f:
    if args.no_lbr:
        print("no_lbr", file=f)
    for etype, expr in exprs:
        if etype == "FDATA":
            issym1, anchor1, offsym1, issym2, anchor2, offsym2, execnt, mispred = expr
            print(
                evaluate_symbol(issym1, anchor1, offsym1),
                evaluate_symbol(issym2, anchor2, offsym2),
                execnt,
                mispred,
                file=f,
            )
        elif etype == "NOLBR":
            issym, anchor, offsym, count = expr
            print(evaluate_symbol(issym, anchor, offsym), count, file=f)
        elif etype == "PREAGG":
            # Replace all symbols enclosed in ##
            print(expr[0], re.sub(replace_pat, replace_symbol, expr[1]), file=f)
        else:
            exit("ERROR: unhandled expression type:\n%s" % etype)



# coding=utf8
"""
Test that the expression parser returns proper Unicode strings.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

# this test case fails because of rdar://12991846
# the expression parser does not deal correctly with Unicode expressions
# e.g.
# (lldb) expr L"Hello"
# (const wchar_t [6]) $0 = {
#  [0] = \0\0\0\0
#  [1] = \0\0\0\0
#  [2] = \0\0\0\0
#  [3] = \0\0\0\0
#  [4] = H\0\0\0
#  [5] = e\0\0\0
# }


class UnicodeLiteralsTestCase(TestBase):
    def test_expr1(self):
        """Test that the expression parser returns proper Unicode strings."""
        self.rdar12991846(expr=1)

    def test_expr2(self):
        """Test that the expression parser returns proper Unicode strings."""
        self.rdar12991846(expr=2)

    def test_expr3(self):
        """Test that the expression parser returns proper Unicode strings."""
        self.rdar12991846(expr=3)

    def rdar12991846(self, expr=None):
        """Test that the expression parser returns proper Unicode strings."""
        if self.getArchitecture() in ["i386"]:
            self.skipTest("Skipping because this test is known to crash on i386")

        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.cpp")
        )

        if expr == 1:
            self.expect('expression L"hello"', substrs=["hello"])

        if expr == 2:
            self.expect('expression u"hello"', substrs=["hello"])

        if expr == 3:
            self.expect('expression U"hello"', substrs=["hello"])



"""
Test that we can set up software breakpoint even if we failed to decode and execute instruction
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBreakpointIllegal(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipIf(archs=no_match(["rv64gc"]))
    def test_4(self):
        self.build()
        (target, process, cur_thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "main", lldb.SBFileSpec("main.c")
        )
        self.runCmd("thread step-inst")
        # we need to step more, as some compilers do not set appropriate debug info.
        while cur_thread.stop_description == "instruction step into":
            self.runCmd("thread step-inst")
        # The stop reason of the thread should be illegal opcode.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_SIGNAL,
            substrs=["stopped", "stop reason = signal SIGILL: illegal opcode"],
        )

    @skipIf(archs=no_match(["rv64gc"]))
    def test_2(self):
        self.build(dictionary={"C_SOURCES": "compressed.c", "EXE": "compressed.x"})
        (target, process, cur_thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "main", lldb.SBFileSpec("compressed.c"), exe_name="compressed.x"
        )
        self.runCmd("thread step-inst")
        # we need to step more, as some compilers do not set appropriate debug info.
        while cur_thread.stop_description == "instruction step into":
            self.runCmd("thread step-inst")
        # The stop reason of the thread should be illegal opcode.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_SIGNAL,
            substrs=["stopped", "stop reason = signal SIGILL: illegal opcode"],
        )



"""
Test that CoreFoundation classes CFGregorianDate and CFRange are not improperly uniqued
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Rdar10967107TestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # We'll use the test method name as the exe_name.
        self.exe_name = self.testMethodName
        # Find the line number to break inside main().
        self.main_source = "main.m"
        self.line = line_number(self.main_source, "// Set breakpoint here.")

    def test_cfrange_diff_cfgregoriandate(self):
        """Test that CoreFoundation classes CFGregorianDate and CFRange are not improperly uniqued."""
        d = {"EXE": self.exe_name}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, self.main_source, self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        # check that each type is correctly bound to its list of children
        self.expect(
            "frame variable cf_greg_date --raw",
            substrs=["year", "month", "day", "hour", "minute", "second"],
        )
        self.expect("frame variable cf_range --raw", substrs=["location", "length"])
        # check that printing both does not somehow confuse LLDB
        self.expect(
            "frame variable  --raw",
            substrs=[
                "year",
                "month",
                "day",
                "hour",
                "minute",
                "second",
                "location",
                "length",
            ],
        )



"""Test that debugserver will parse a mach-o in inferior memory even if it's not loaded."""

import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestUnregisteredMacho(TestBase):
    # newer debugserver required for jGetLoadedDynamicLibrariesInfos
    # to support this
    @no_debug_info_test
    @skipUnlessDarwin
    def test(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        frame = thread.GetFrameAtIndex(0)
        macho_buf = frame.GetValueForVariablePath("macho_buf")
        macho_addr = macho_buf.GetValueAsUnsigned()
        invalid_macho_addr = macho_buf.GetValueAsUnsigned() + 4
        gdb_packet = (
            "process plugin packet send 'jGetLoadedDynamicLibrariesInfos:{\"solib_addresses\":[%d]}]'"
            % macho_addr
        )

        # Send the jGetLoadedDynamicLibrariesInfos packet
        # to debugserver, asking it to parse the mach-o binary
        # at this address and give us the UUID etc, even though
        # dyld doesn't think there is a binary at that address.
        # We won't get a pathname for the binary (from dyld), but
        # we will get to the LC_UUID and include that.
        self.expect(
            gdb_packet,
            substrs=['"pathname":""', '"uuid":"1B4E28BA-2FA1-11D2-883F-B9A761BDE3FB"'],
        )

        no_macho_gdb_packet = (
            "process plugin packet send 'jGetLoadedDynamicLibrariesInfos:{\"solib_addresses\":[%d]}]'"
            % invalid_macho_addr
        )
        self.expect(no_macho_gdb_packet, substrs=['response: {"images":[]}'])

        # Test that we get back the information for the properly
        # formatted Mach-O binary in memory, but do not get an
        # entry for the invalid Mach-O address.
        both_gdb_packet = (
            "process plugin packet send 'jGetLoadedDynamicLibrariesInfos:{\"solib_addresses\":[%d,%d]}]'"
            % (macho_addr, invalid_macho_addr)
        )
        self.expect(both_gdb_packet, substrs=['"load_address":%d,' % macho_addr])
        self.expect(
            both_gdb_packet,
            substrs=['"load_address":%d,' % invalid_macho_addr],
            matching=False,
        )



"""
Test the memory commands operating on memory regions with holes (inaccessible
pages)
"""

import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *


class MemoryHolesTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        super().setUp()
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// break here")

    def _prepare_inferior(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main() after the variables are assigned values.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # Avoid the expression evaluator, as it can allocate allocate memory
        # inside the holes we've deliberately left empty.
        self.memory = self.frame().FindVariable("mem_with_holes").GetValueAsUnsigned()
        self.pagesize = self.frame().FindVariable("pagesize").GetValueAsUnsigned()
        self.num_pages = (
            self.target().FindFirstGlobalVariable("num_pages").GetValueAsUnsigned()
        )
        positions = self.frame().FindVariable("positions")
        self.positions = [
            positions.GetChildAtIndex(i).GetValueAsUnsigned()
            for i in range(positions.GetNumChildren())
        ]
        self.assertEqual(len(self.positions), 5)

    def test_memory_read(self):
        self._prepare_inferior()

        error = lldb.SBError()
        content = self.process().ReadMemory(self.memory, 2 * self.pagesize, error)
        self.assertEqual(len(content), self.pagesize)
        self.assertEqual(content[0:7], b"needle\0")
        self.assertTrue(error.Fail())

    def test_memory_find(self):
        self._prepare_inferior()

        matches = [f"data found at location: {p:#x}" for p in self.positions]
        self.expect(
            f'memory find --count {len(self.positions)+1} --string "needle" '
            f"{self.memory:#x} {self.memory+self.pagesize*self.num_pages:#x}",
            substrs=matches + ["no more matches within the range"],
        )



"""
Test sending SIGINT Process IOHandler
"""

import os

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestCase(PExpectTest):
    SHARED_BUILD_TESTCASE = False

    @skipIf(macos_version=["<", "14.0"], asan=True)
    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])
    def test(self):
        self.build(dictionary={"CXX_SOURCES": "cat.cpp"})
        self.launch(executable=self.getBuildArtifact())

        self.child.sendline("process launch")
        self.child.expect("Process .* launched")

        self.child.sendline("Hello cat")
        self.child.expect_exact("read: Hello cat")

        self.child.sendintr()
        self.child.expect("Process .* stopped")
        self.expect_prompt()

        self.expect("bt", substrs=["input_copy_loop"])

        self.child.sendline("continue")
        self.child.expect("Process .* resuming")

        self.child.sendline("Goodbye cat")
        self.child.expect_exact("read: Goodbye cat")

        self.child.sendeof()
        self.child.expect("Process .* exited")
        self.expect_prompt()

        self.quit()



"""
Test that suspended threads do not affect should-stop decisions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class IgnoreSuspendedThreadTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers for our breakpoints.
        self.break_1 = line_number("main.cpp", "// Set first breakpoint here")
        self.break_2 = line_number("main.cpp", "// Set second breakpoint here")
        self.break_3 = line_number("main.cpp", "// Set third breakpoint here")

    def printThreadsStoppedByBreakpoint(self, process):
        stopped_threads = lldbutil.get_stopped_threads(
            process, lldb.eStopReasonBreakpoint
        )
        for thread in stopped_threads:
            break_id = thread.GetStopReasonDataAtIndex(0)
            print(
                "Thread "
                + str(thread.GetThreadID())
                + " stopped at breakpoint "
                + str(break_id)
            )

    def test(self):
        self.build()
        target = lldbutil.run_to_breakpoint_make_target(self)

        # This should create a breakpoint with 1 location.
        bp1_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )

        bp2_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_2, num_expected_locations=1
        )

        bp3_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_3, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)
        # Get the target process
        process = target.GetProcess()

        if self.TraceOn():
            print("First stop:")
            self.printThreadsStoppedByBreakpoint(process)

        thread_to_suspend = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp1_id
        )
        self.assertIsNotNone(thread_to_suspend, "Should hit breakpoint 1")
        thread_to_suspend.Suspend()

        # Do not stop at bp2 and autocontinue to bp3
        target.FindBreakpointByID(bp2_id).SetAutoContinue(True)

        # Run to the third breakpoint
        self.runCmd("continue")

        if self.TraceOn():
            print("Second stop:")
            self.printThreadsStoppedByBreakpoint(process)

        stopped_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp3_id
        )
        self.assertIsNotNone(stopped_thread, "Should hit breakpoint 3")

        thread_to_suspend.Resume()

        # Run to completion
        self.runCmd("continue")

        # At this point, the inferior process should have exited.
        self.assertEqual(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



"""Test SBValue::Persist"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SBValuePersistTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        """Test SBValue::Persist"""
        self.build()
        self.setTearDownCleanup()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_source_regexp(self, "break here")

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synthetic clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        foo = self.frame().FindVariable("foo")
        bar = self.frame().FindVariable("bar")
        baz = self.frame().FindVariable("baz")

        self.assertTrue(foo.IsValid(), "foo is not valid")
        self.assertTrue(bar.IsValid(), "bar is not valid")
        self.assertTrue(baz.IsValid(), "baz is not valid")

        fooPersist = foo.Persist()
        barPersist = bar.Persist()
        bazPersist = baz.Persist()

        self.assertTrue(fooPersist.IsValid(), "fooPersist is not valid")
        self.assertTrue(barPersist.IsValid(), "barPersist is not valid")
        self.assertTrue(bazPersist.IsValid(), "bazPersist is not valid")

        self.assertEqual(fooPersist.GetValueAsUnsigned(0), 10, "fooPersist != 10")
        self.assertEqual(barPersist.GetPointeeData().sint32[0], 4, "barPersist != 4")
        self.assertEqual(bazPersist.GetSummary(), '"85"', "bazPersist != 85")

        self.runCmd("continue")

        self.assertTrue(fooPersist.IsValid(), "fooPersist is not valid")
        self.assertTrue(barPersist.IsValid(), "barPersist is not valid")
        self.assertTrue(bazPersist.IsValid(), "bazPersist is not valid")

        self.assertEqual(fooPersist.GetValueAsUnsigned(0), 10, "fooPersist != 10")
        self.assertEqual(barPersist.GetPointeeData().sint32[0], 4, "barPersist != 4")
        self.assertEqual(bazPersist.GetSummary(), '"85"', "bazPersist != 85")

        self.expect("expr *(%s)" % (barPersist.GetName()), substrs=["= 4"])



"""
Test clang module build progress events.
"""
import os
import shutil

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestCase(TestBase):
    @skipUnlessDarwin
    def test_clang_module_build_progress_report(self):
        """Test receipt of progress events for clang module builds"""
        self.build()

        # Ensure an empty module cache.
        mod_cache = self.getBuildArtifact("new-modules")
        if os.path.isdir(mod_cache):
            shutil.rmtree(mod_cache)
        self.runCmd(f"settings set symbols.clang-modules-cache-path '{mod_cache}'")

        # TODO: The need for this seems like a bug.
        self.runCmd(
            f"settings set target.clang-module-search-paths '{self.getSourceDir()}'"
        )

        lldbutil.run_to_name_breakpoint(self, "main")

        # Just before triggering module builds, start listening for progress
        # events. Listening any earlier would result in a queue filled with
        # other unrelated progress events.
        broadcaster = self.dbg.GetBroadcaster()
        listener = lldbutil.start_listening_from(
            broadcaster, lldb.SBDebugger.eBroadcastBitProgress
        )

        # Trigger module builds.
        self.expect("expression @import MyModule")

        event = lldbutil.fetch_next_event(self, listener, broadcaster)
        payload = lldb.SBDebugger.GetProgressFromEvent(event)
        message = payload[0]
        self.assertEqual(message, "Building Clang modules")



import lldb
import struct

from lldb.plugins.operating_system import OperatingSystem


class OperatingSystemPlugIn(OperatingSystem):
    """Class that provides data for an instance of a LLDB 'OperatingSystemPython' plug-in class"""

    def __init__(self, process):
        super().__init__(process)

    def create_thread(self, tid, context):
        if tid == 0x444444444:
            thread_info = {
                "tid": tid,
                "name": "four",
                "queue": "queue4",
                "state": "stopped",
                "stop_reason": "none",
            }
            self.threads.append(thread_info)
            return thread_info
        return None

    def get_thread_info(self):
        if self.process.state != lldb.eStateStopped:
            print("Error: get_thread_info called with state not stopped")
            return []

        if not self.threads:
            self.threads = [
                {
                    "tid": 0x111111111,
                    "name": "one",
                    "queue": "queue1",
                    "state": "stopped",
                    "stop_reason": "breakpoint",
                },
                {
                    "tid": 0x222222222,
                    "name": "two",
                    "queue": "queue2",
                    "state": "stopped",
                    "stop_reason": "none",
                },
                {
                    "tid": 0x333333333,
                    "name": "three",
                    "queue": "queue3",
                    "state": "stopped",
                    "stop_reason": "trace",
                },
            ]
        return self.threads

    def get_register_info(self):
        if self.registers is None:
            self.registers = dict()
            self.registers["sets"] = ["GPR"]
            self.registers["registers"] = [
                {
                    "name": "rax",
                    "bitsize": 64,
                    "offset": 0,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 0,
                    "dwarf": 0,
                },
                {
                    "name": "rbx",
                    "bitsize": 64,
                    "offset": 8,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 3,
                    "dwarf": 3,
                },
                {
                    "name": "rcx",
                    "bitsize": 64,
                    "offset": 16,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 2,
                    "dwarf": 2,
                    "generic": "arg4",
                    "alt-name": "arg4",
                },
                {
                    "name": "rdx",
                    "bitsize": 64,
                    "offset": 24,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 1,
                    "dwarf": 1,
                    "generic": "arg3",
                    "alt-name": "arg3",
                },
                {
                    "name": "rdi",
                    "bitsize": 64,
                    "offset": 32,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 5,
                    "dwarf": 5,
                    "generic": "arg1",
                    "alt-name": "arg1",
                },
                {
                    "name": "rsi",
                    "bitsize": 64,
                    "offset": 40,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 4,
                    "dwarf": 4,
                    "generic": "arg2",
                    "alt-name": "arg2",
                },
                {
                    "name": "rbp",
                    "bitsize": 64,
                    "offset": 48,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 6,
                    "dwarf": 6,
                    "generic": "fp",
                    "alt-name": "fp",
                },
                {
                    "name": "rsp",
                    "bitsize": 64,
                    "offset": 56,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 7,
                    "dwarf": 7,
                    "generic": "sp",
                    "alt-name": "sp",
                },
                {
                    "name": "r8",
                    "bitsize": 64,
                    "offset": 64,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 8,
                    "dwarf": 8,
                    "generic": "arg5",
                    "alt-name": "arg5",
                },
                {
                    "name": "r9",
                    "bitsize": 64,
                    "offset": 72,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 9,
                    "dwarf": 9,
                    "generic": "arg6",
                    "alt-name": "arg6",
                },
                {
                    "name": "r10",
                    "bitsize": 64,
                    "offset": 80,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 10,
                    "dwarf": 10,
                },
                {
                    "name": "r11",
                    "bitsize": 64,
                    "offset": 88,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 11,
                    "dwarf": 11,
                },
                {
                    "name": "r12",
                    "bitsize": 64,
                    "offset": 96,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 12,
                    "dwarf": 12,
                },
                {
                    "name": "r13",
                    "bitsize": 64,
                    "offset": 104,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 13,
                    "dwarf": 13,
                },
                {
                    "name": "r14",
                    "bitsize": 64,
                    "offset": 112,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 14,
                    "dwarf": 14,
                },
                {
                    "name": "r15",
                    "bitsize": 64,
                    "offset": 120,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 15,
                    "dwarf": 15,
                },
                {
                    "name": "rip",
                    "bitsize": 64,
                    "offset": 128,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "gcc": 16,
                    "dwarf": 16,
                    "generic": "pc",
                    "alt-name": "pc",
                },
                {
                    "name": "rflags",
                    "bitsize": 64,
                    "offset": 136,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                    "generic": "flags",
                    "alt-name": "flags",
                },
                {
                    "name": "cs",
                    "bitsize": 64,
                    "offset": 144,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                },
                {
                    "name": "fs",
                    "bitsize": 64,
                    "offset": 152,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                },
                {
                    "name": "gs",
                    "bitsize": 64,
                    "offset": 160,
                    "encoding": "uint",
                    "format": "hex",
                    "set": 0,
                },
            ]
        return self.registers

    def get_register_data(self, tid):
        return struct.pack(
            "21Q",
            tid + 1,
            tid + 2,
            tid + 3,
            tid + 4,
            tid + 5,
            tid + 6,
            tid + 7,
            tid + 8,
            tid + 9,
            tid + 10,
            tid + 11,
            tid + 12,
            tid + 13,
            tid + 14,
            tid + 15,
            tid + 16,
            tid + 17,
            tid + 18,
            tid + 19,
            tid + 20,
            tid + 21,
        )



import dap_server
import json
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase


class TestDAP_redirection_to_console(lldbdap_testcase.DAPTestCaseBase):
    def test(self):
        """
        Without proper stderr and stdout redirection, the following code would throw an
        exception, like the following:

            Exception: unexpected malformed message from lldb-dap
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program, lldbDAPEnv={"LLDB_DAP_TEST_STDOUT_STDERR_REDIRECTION": ""}
        )

        source = "main.cpp"

        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint_ids = self.set_source_breakpoints(source, [breakpoint1_line])

        self.assertEqual(len(breakpoint_ids), 1, "expect correct number of breakpoints")
        self.continue_to_breakpoints(breakpoint_ids)

        self.assertIn(
            "argc", json.dumps(self.dap_server.get_local_variables(frameIndex=1))
        )



import os
import shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestClangModuleHashMismatch(TestBase):
    @add_test_categories(["gmodules"])
    def test_expr(self):
        with open(self.getBuildArtifact("module.modulemap"), "w") as f:
            f.write(
                """
                    module Foo { header "f.h" }
                    """
            )
        with open(self.getBuildArtifact("f.h"), "w") as f:
            f.write(
                """
                    typedef int my_int;
                    void f() {}
                    """
            )

        mod_cache = self.getBuildArtifact("private-module-cache")
        if os.path.isdir(mod_cache):
            shutil.rmtree(mod_cache)
        self.build()
        self.assertTrue(os.path.isdir(mod_cache), "module cache exists")

        logfile = self.getBuildArtifact("host.log")
        with open(logfile, "w") as f:
            sbf = lldb.SBFile(f.fileno(), "w", False)
            status = self.dbg.SetErrorFile(sbf)
            self.assertSuccess(status)

            target, _, _, _ = lldbutil.run_to_source_breakpoint(
                self, "break here", lldb.SBFileSpec("main.m")
            )
            target.GetModuleAtIndex(0).FindTypes("my_int")

        with open(logfile, "r") as f:
            for line in f:
                if "hash mismatch" in line and "Foo" in line:
                    found = True
        self.assertTrue(found)



"""Test that AArch64 PAC bits are stripped from address expression arguments"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrauthAddressExpressions(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # On Darwin systems, arch arm64e means ARMv8.3 with ptrauth
    # ABI used.
    @skipIf(archs=no_match(["arm64e"]))
    def test(self):
        # Skip this test if not running on AArch64 target that supports PAC
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication.")
        self.source = "main.c"
        self.build()
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source, False)
        )

        self.expect("p fptr", substrs=[self.source])
        self.expect("ima loo -va fptr", substrs=[self.source])
        self.expect("break set -a fptr", substrs=[self.source])



"""
Test number of threads.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BreakpointAfterJoinTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number for our breakpoint.
        self.breakpoint = line_number("main.cpp", "// Set breakpoint here")

    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="llvm.org/pr15824 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=lldbplatformutil.getDarwinOSTriples(),
        bugnumber="llvm.org/pr15824 thread states not properly maintained and <rdar://problem/28557237>",
    )
    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureNetBSD
    def test(self):
        """Test breakpoint handling after a thread join."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.breakpoint, num_expected_locations=1
        )

        # The breakpoint list should show 1 location.
        self.expect(
            "breakpoint list -f",
            "Breakpoint location shown correctly",
            substrs=[
                "1: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.breakpoint
            ],
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        # The exit probably occurred during breakpoint handling, but it isn't
        # guaranteed.  The main thing we're testing here is that the debugger
        # handles this cleanly is some way.

        # Get the number of threads
        num_threads = process.GetNumThreads()

        # Make sure we see at least six threads
        self.assertGreaterEqual(
            num_threads,
            6,
            "Number of expected threads and actual threads do not match.",
        )

        # Make sure all threads are stopped
        for i in range(0, num_threads):
            self.assertTrue(
                process.GetThreadAtIndex(i).IsStopped(),
                "Thread {0} didn't stop during breakpoint.".format(i),
            )

        # Run to completion
        self.runCmd("continue")

        # If the process hasn't exited, collect some information
        if process.GetState() != lldb.eStateExited:
            self.runCmd("thread list")
            self.runCmd("process status")

        # At this point, the inferior process should have exited.
        self.assertEqual(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )

        self.dbg.HandleCommand(
            f"type summary add --expand -s 'some summary' SummaryAndChildren"
        )
        self.dbg.HandleCommand(f"type summary add -s 'some summary' SummaryOnly")

        self.expect(
            "v summary_and_children_ref", substrs=["some summary", "child = 30"]
        )
        self.expect(
            "v summary_only_ref", patterns=["some summary", "(?s)(?!.*child = )"]
        )
        self.expect(
            "v children_only_ref", patterns=["(?s)(?!.*some summary)", "child = 30"]
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestCase(TestBase):
    def test_invalid_arg(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect(
            "thread select 0x1ffffffff",
            error=True,
            startstr="error: Invalid thread index '0x1ffffffff'",
        )
        self.expect(
            "thread select -t 0x1ffffffff",
            error=True,
            startstr="error: Invalid thread ID",
        )
        self.expect(
            "thread select 1 2 3",
            error=True,
            startstr="error: 'thread select' takes exactly one thread index argument, or a thread ID option:",
        )
        self.expect(
            "thread select -t 1234 1",
            error=True,
            startstr="error: 'thread select' cannot take both a thread ID option and a thread index argument:",
        )
        # Parses but not a valid thread id.
        self.expect(
            "thread select 0xffffffff",
            error=True,
            startstr="error: Invalid thread index #0xffffffff.",
        )
        self.expect(
            "thread select -t 0xffffffff",
            error=True,
            startstr="error: Invalid thread ID",
        )

    def test_thread_select_tid(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        self.runCmd(
            "thread select -t %d" % self.thread().GetThreadID(),
        )



from lldbsuite.test import lldbinline
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbplatformutil

supported_archs = ["x86_64", "aarch64", "arm64", "arm64e"]
decorators = [
    skipIf(archs=no_match(supported_archs)),
    skipIf(compiler="clang", compiler_version=["<", "11.0"]),
    skipUnlessHasCallSiteInfo,
    skipIf(dwarf_version=["<", "4"]),
]

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    decorators=decorators,
    name="BasicEntryValues_V5",
    build_dict=dict(CXXFLAGS_EXTRAS="-O2 -glldb"),
)

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    decorators=decorators + [skipIf(debug_info="dsym")],
    name="BasicEntryValues_GNU",
    build_dict=dict(CXXFLAGS_EXTRAS="-O2 -ggdb -gdwarf-4"),
)
